SystemOrganization addCategory: #'Fuel-Collections'!
SystemOrganization addCategory: #'Fuel-Core'!
SystemOrganization addCategory: #'Fuel-Core-Clusters'!
SystemOrganization addCategory: #'Fuel-Core-Clusters-Optionals'!
SystemOrganization addCategory: #'Fuel-Core-Errors'!
SystemOrganization addCategory: #'Fuel-Core-Mappers'!
SystemOrganization addCategory: #'Fuel-Streams'!
SystemOrganization addCategory: #'Fuel-Utilities'!

!ByteSymbol class methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 01:53'!
materializeFrom: aReadStream

	^ aReadStream nextString asSymbol! !

!ByteSymbol methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:15'!
fuelAccept: aVisitor

	^aVisitor visitSymbol: self! !

!ByteSymbol methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 01:52'!
serializeOn: aWriteStream

	aWriteStream nextStringPut: self! !

!Set methodsFor: '*Fuel' stamp: 'MartinDias 9/9/2011 14:46'!
addIfNotPresent: anObject ifPresentDo: aBlock
	"Include anObject as one of the receiver's elements and then value aBlock, but only if there
	is no such element already. Anwser anObject."

	| index |
	index := self scanFor: anObject.
	(array at: index) 
		ifNil: [self atNewIndex: index put: anObject asSetElement]
		ifNotNil: [ aBlock value ].
	^ anObject

	"Sets in Pharo 1.2 can't' contain nil, so this implementation is wrong in that version. This is the right one (in Pharo 1.2):

	| index |
	anObject ifNil: [self error: 'Sets cannot meaningfully contain nil as an element'].
	index := self findElementOrNil: anObject.
	(array at: index) 
		ifNil: [ self atNewIndex: index put: anObject ]
		ifNotNil: [ aBlock value ].
	^ anObject"! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 12/3/2011 19:39'!
abstractBytecodeMessagesFrom: startpc to: endpc
	"Answer an OrderedCollection of the abstract bytecodes sent in the receiver."
	| scanner abstractBytecodes |
	scanner := InstructionStream new method: self pc: startpc.
	"now collect all the bytecode messages in the block."
	abstractBytecodes := OrderedCollection new.
	[scanner pc <= endpc] whileTrue:
		[[scanner interpretNextInstructionFor: nil]
			on: MessageNotUnderstood
			do: [:ex| abstractBytecodes add: ex message]].
	^abstractBytecodes

	"| m | (m := CompiledMethod >> #abstractBytecodeMessagesFrom:to:) abstractBytecodeMessagesFrom: m initialPC to: m endPC"! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:13'!
fuelAccept: aVisitor

	^aVisitor visitCompiledMethod: self! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 10/22/2010 12:56'!
isNamedPrimitive 

	^ self primitive = 117! !

!Character class methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 15:42'!
materializeFrom: aReadStream

	^self value: aReadStream next! !

!Character methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:17'!
fuelAccept: aVisitor

	^aVisitor visitCharacter: self! !

!Character methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 15:41'!
serializeOn: aWriteStream

	aWriteStream nextPut: value! !

!TimeStamp methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 15:39'!
fuelAccept: aVisitor

	^ aVisitor visitFixedObject: self
! !

!ByteArray methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:12'!
fuelAccept: aVisitor

	^aVisitor visitBytesObject: self! !

Error subclass: #FLMaterializationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

FLMaterializationError subclass: #FLBadSignature
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!FLBadSignature commentStamp: 'MarianoMartinezPeck 10/23/2011 14:32' prior: 0!
I represent an error produced during materialization when the serialized signature doesn't match the materializer's signature (accessible via FLMaterializer>>signature). A signature is a byte prefix that should prefix a well-serialized stream.!

!FLBadSignature class methodsFor: 'signaling' stamp: 'MarianoMartinezPeck 10/23/2011 14:23'!
signalCurrentSignature: currentSignature streamSignature: streamSignature
	self signal: 'Current signature is ', currentSignature asString, ' but the signature in the stream is ', streamSignature asString! !

FLMaterializationError subclass: #FLBadVersion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!FLBadVersion commentStamp: 'MarianoMartinezPeck 10/23/2011 14:33' prior: 0!
I represent an error produced during materialization when the serialized version doesn't match the materializer's version (accessible via FLMaterializer>>version). A version is encoded in 16 bits and is enconded heading the serialized stream, after the signature.!

!FLBadVersion class methodsFor: 'signaling' stamp: 'MarianoMartinezPeck 10/23/2011 14:20'!
signalCurrentVersion: currentVersion streamVersion: streamVersion
	^ self signal: 'Current version is ', currentVersion asString, ' but the version in the stream is ', streamVersion asString ! !

FLMaterializationError subclass: #FLClassNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!FLClassNotFound commentStamp: 'MarianoMartinezPeck 10/23/2011 14:34' prior: 0!
I represent an error produced during materialization when a serialized class or trait name doesn't exist.!

!FLClassNotFound class methodsFor: 'signaling' stamp: 'MartinDias 10/24/2011 17:56'!
signalWithName: className
	^ self signal: 'Class ', className printString, ' not found at materialization'! !

FLMaterializationError subclass: #FLGlobalNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!FLGlobalNotFound commentStamp: 'MartinDias 12/16/2011 01:16' prior: 0!
I represent an error produced during materialization when a serialized global name doesn't exist (at Smalltalk globals).!

!FLGlobalNotFound class methodsFor: 'signaling' stamp: 'MartinDias 10/24/2011 17:56'!
signalWithName: aName
	^ self signal: 'Global ', aName printString, ' not found at materialization'! !

FLMaterializationError subclass: #FLMethodNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!FLMethodNotFound commentStamp: 'MartinDias 12/16/2011 01:17' prior: 0!
I represent an error produced during materialization when a serialized method in a class or trait name doesn't exist (at Smalltalk globals).!

!FLMethodNotFound class methodsFor: 'signaling' stamp: 'MartinDias 11/29/2011 20:58'!
signalWith: aGlobalName and: aSelector
	^ self signal: 'Method ', aGlobalName, '>>', aSelector, ' not found at materialization'! !

Error subclass: #FLSerializationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

FLSerializationError subclass: #FLObsolete
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!FLObsolete class methodsFor: 'signaling' stamp: 'MartinDias 12/16/2011 01:32'!
signalWithName: classOrTraitName
	^ self signal: classOrTraitName printString, ' can not be serialized.'! !

!WriteStream methodsFor: '*Fuel' stamp: 'MartinDias 8/13/2011 14:28'!
nextBytesPutAll: aCollection
	"Append the bytes of aCollection to the sequence of bytes accessible 
	by the receiver. Answer aCollection."
 
 	| newEnd |
 	collection class instSpec == aCollection class instSpec ifFalse:
 		[^ super nextPutAll: aCollection ].
 
 	newEnd := position + aCollection size.
 	newEnd > writeLimit ifTrue:
 		[self growTo: newEnd + 10].
 
 	collection replaceFrom: position+1 to: newEnd  with: aCollection startingAt: 1.
 	position := newEnd.
	
	^ aCollection! !

!StandardFileStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/6/2011 12:42'!
fuelNextWordsInto: aWordObject
	"This method is the same as nextWordsInto: but the restoreEndianness is only done if needed"
	self next: aWordObject basicSize into: aWordObject startingAt: 1.
	^ aWordObject! !

!StandardFileStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 10/10/2011 12:24'!
nextBytesPutAll: aCollection
	self nextPutAll: aCollection! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 6/13/2011 01:04'!
fuelIgnoredInstanceVariableNames
	"Indicates which variables have to be ignored during serialization."

	^#()! !

!UndefinedObject methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:36'!
fuelAccept: aVisitor

	^aVisitor visitWellKnownObject: self! !

!PositionableStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/6/2011 17:02'!
fuelNextWordsInto: aWordObject 
	"This method is the same as nextWordsInto: but the restoreEndianness is only done if needed"
	| blt pos source byteSize |

	byteSize := aWordObject byteSize.
	"is the test on collection basicSize \\ 4 necessary?"
	
	((self position bitAnd: 3) = 0 and: [ (collection basicSize bitAnd: 3) = 0])
		ifTrue: [source := collection.
			pos := self position.
			self skip: byteSize]
		ifFalse: ["forced to copy it into a buffer"
			source := self next: byteSize.
			pos := 0].

	"Now use BitBlt to copy the bytes to the bitmap."
	blt := (BitBlt current
				toForm: (Form new hackBits: aWordObject))
				sourceForm: (Form new hackBits: source).
	blt combinationRule: Form over. "store"
	blt sourceX: 0;
		 sourceY: pos // 4;
		 height: byteSize // 4;
		 width: 4.
	blt destX: 0;
		 destY: 0.
	blt copyBits.

	^ aWordObject 	
! !

!PositionableStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 10/6/2011 21:16'!
int64
	"Answer the next signed, 32-bit integer from this (binary) stream."
	"Details: As a fast check for negative number, check the high bit of the first digit"

	| n firstDigit |
	n := firstDigit := self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	firstDigit >= 128 ifTrue: [n := -16r10000000000000000 + n].  "decode negative 64-bit integer"
	^ n
! !

!PositionableStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 10/6/2011 21:16'!
int8
	| val |
	val := self next.
	val >= 128 ifTrue: [^(256 - val) negated].
	^val! !

!PositionableStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 10/6/2011 21:17'!
uint64
	"Answer the next unsigned, 64-bit integer from this (binary) stream."

	| n |
	n := self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	^ n
! !

!PositionableStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 10/6/2011 20:31'!
uint64: anInteger
	"Store the given unsigned, 64-bit integer on this (binary) stream."

	(anInteger < 0) | (anInteger >= 16r10000000000000000)
		ifTrue: [self error: 'outside unsigned 64-bit integer range'].
		
	self nextPut: (anInteger digitAt: 8).
	self nextPut: (anInteger digitAt: 7).
	self nextPut: (anInteger digitAt: 6).
	self nextPut: (anInteger digitAt: 5).
	self nextPut: (anInteger digitAt: 4).
	self nextPut: (anInteger digitAt: 3).
	self nextPut: (anInteger digitAt: 2).
	self nextPut: (anInteger digitAt: 1).
! !

!PositionableStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 10/6/2011 21:15'!
uint8
	"Answer the next unsigned, 16-bit integer from this (binary) stream."

	^ self next.
! !

!BlockClosure methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 12/3/2011 19:39'!
abstractBytecodes
	"Answer an OrderedCollection of the abstract bytecodes sent in the receiver."
	| scanner blockCreationMessage |
	scanner := InstructionStream new method: self method pc: self method initialPC.
	[scanner pc < startpc] whileTrue:
		[[scanner interpretNextInstructionFor: nil]
			on: MessageNotUnderstood
			do: [:ex| blockCreationMessage := ex message]].
	^self method abstractBytecodeMessagesFrom: startpc to: blockCreationMessage arguments last + startpc - 1

	"(SortedCollection sortBlock: [:a :b| a compare: b caseSensitive: false]) sortBlock abstractBytecodes"! !

!BlockClosure methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 12/3/2011 00:15'!
fuelAccept: aVisitor

	^aVisitor visitBlockClosure: self! !

!BlockClosure methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 12/3/2011 19:40'!
isClean
	^((self abstractBytecodes collect: [:ea| ea selector]) intersection: #(pushReceiver pushReceiverVariable: popIntoReceiverVariable: storeIntoReceiverVariable: methodReturnConstant: methodReturnReceiver methodReturnTop)) isEmpty
	
		"A clean closure is one that doesn't really need the home context because:
		- It doesn't send messages to self or super
		- It doesn't access any instance variable
		- It doesn't access any outer temp
		- It doesn't do ^ return"
	
	"[^nil] isClean"
	"[:a :b| a < b] isClean"! !

!BlockClosure methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 12/3/2011 00:12'!
outerContext: aContext startpc: aStartpc numArgs: argCount
	outerContext := aContext.
	startpc := aStartpc.
	numArgs := argCount.! !

!Collection methodsFor: '*Fuel' stamp: 'MartinDias 5/19/2011 23:41'!
addIfNotPresent: anObject ifPresentDo: aBlock
	"Include anObject as one of the receiver's elements and then value aBlock, but only if there
	is no such element already. Anwser anObject."

	(self includes: anObject) 
		ifFalse: [ self add: anObject ]
		ifTrue: [ aBlock value ].
	^ anObject! !

Collection subclass: #LargeIdentityHashedCollection
	instanceVariableNames: 'tally tallies'
	classVariableNames: 'PermutationMap'
	poolDictionaries: ''
	category: 'Fuel-Collections'!

LargeIdentityHashedCollection subclass: #LargeIdentityDictionary
	instanceVariableNames: 'keys values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Collections'!

!LargeIdentityDictionary methodsFor: 'accessing' stamp: 'ul 12/18/2011 11:21'!
at: key

	| hash |
	(keys at: (hash := key largeIdentityHash + 1)) ifNotNil: [ :keyList | 
		1 to: (tallies at: hash) do: [ :index |
			(keyList at: index) == key ifTrue: [
				^(values at: hash) at: index ] ] ].
	self errorKeyNotFound: key! !

!LargeIdentityDictionary methodsFor: 'accessing' stamp: 'ul 12/18/2011 11:22'!
at: key ifAbsent: aBlock

	| hash |
	(keys at: (hash := key largeIdentityHash + 1)) ifNotNil: [ :keyList | 
		1 to: (tallies at: hash) do: [ :index |
			(keyList at: index) == key ifTrue: [
				^(values at: hash) at: index ] ] ].
	^aBlock value! !

!LargeIdentityDictionary methodsFor: 'adding' stamp: 'ul 12/18/2011 11:21'!
at: key put: value

	| hash |
	(keys at: (hash := key largeIdentityHash + 1))
		ifNil: [ 
			keys at: hash put: (Array with: key).
			values at: hash put: (Array with: value).
			tallies at: hash put: 1.
			tally := tally + 1 ]
		ifNotNil: [ :keyList | 
			| newIndex |
			1 to: (tallies at: hash) do: [ :index |
				(keyList at: index) == key ifTrue: [
					^(values at: hash) at: index put: value ] ].
			tally := tally + 1.
			keyList size < (newIndex := (tallies at: hash) + 1)
				ifFalse: [ 
					keyList at: newIndex put: key.
					(values at: hash) at: newIndex put: value ]
				ifTrue: [
					| newList |
					(newList := Array new: keyList size * 2)
						replaceFrom: 1 
						to: keyList size
						with: keyList
						startingAt: 1;
						at: newIndex put: key.
					keys at: hash put: newList.
					(newList := Array new: keyList size * 2)
						replaceFrom: 1 
						to: keyList size
						with: (values at: hash)
						startingAt: 1;
						at: newIndex put: value.
					values at: hash put: newList ].
			tallies at: hash put: newIndex ].
	^value! !

!LargeIdentityDictionary methodsFor: 'enumerating' stamp: 'ul 3/24/2010 21:47'!
do: aBlock

	^self valuesDo: aBlock! !

!LargeIdentityDictionary methodsFor: 'testing' stamp: 'ul 12/18/2011 11:22'!
includesKey: key

	| hash |
	^(keys at: (hash := key largeIdentityHash + 1))
		ifNil: [ false ]
		ifNotNil: [ :keyList | 
			key 
				ifNotNil: [ keyList pointsTo: key ]
				ifNil: [
					1 to: (tallies at: hash) do: [ :index |
						(keyList at: index) == key ifTrue: [ ^true ] ].
					false ] ]! !

!LargeIdentityDictionary methodsFor: 'initialize-release' stamp: 'ul 12/18/2011 11:31'!
initialize

	super initialize.
	keys := Array new: 4096.
	values := Array new: 4096! !

!LargeIdentityDictionary methodsFor: 'accessing' stamp: 'ul 3/24/2010 21:57'!
keys

	^Array new: tally streamContents: [ :stream |
		self keysDo: [ :key | stream nextPut: key ] ]! !

!LargeIdentityDictionary methodsFor: 'enumerating' stamp: 'ul 3/24/2010 21:35'!
keysAndValuesDo: aBlock

	tally = 0 ifTrue: [ ^self ].
	1 to: 4096 do: [ :arrayIndex |
		| keyArray valueArray |
		keyArray := keys at: arrayIndex.
		valueArray := values at: arrayIndex.
		1 to: (tallies at: arrayIndex) do: [ :index |
			aBlock
				value: (keyArray at: index)
				value: (valueArray at: index) ] ]! !

!LargeIdentityDictionary methodsFor: 'enumerating' stamp: 'ul 3/24/2010 21:45'!
keysDo: aBlock

	tally = 0 ifTrue: [ ^self ].
	1 to: 4096 do: [ :arrayIndex |
		| keyArray |
		keyArray := keys at: arrayIndex.
		1 to: (tallies at: arrayIndex) do: [ :index |
			aBlock value: (keyArray at: index) ] ]! !

!LargeIdentityDictionary methodsFor: 'copying' stamp: 'ul 12/18/2011 11:35'!
postCopy

	super postCopy.
	keys := keys copy.
	values := values copy.
	1 to: 4096 do: [ :index |
		(keys at: index) ifNotNil: [ :list |
			keys at: index put: list copy.
			values at: index put: (values at: index) copy ] ]! !

!LargeIdentityDictionary methodsFor: 'removing' stamp: 'ul 12/18/2011 11:22'!
removeKey: key ifAbsent: aBlock

	| hash |
	(keys at: (hash := key largeIdentityHash + 1)) ifNotNil: [ :keyList |
		| size |
		1 to: (size := tallies at: hash) do: [ :index |
			(keyList at: index) == key ifTrue: [
				| valueList |
				keyList
					at: index put: (keyList at: size);
					at: size put: nil.
				(valueList := values at: hash)
					at: index put: (valueList at: size);
					at: size put: nil.
				(size := size - 1) < (keyList size // 4) ifTrue: [
					| newList |
					newList := Array new: size * 2.
					newList
						replaceFrom: 1
						to: size
						with: keyList
						startingAt: 1.
					keys at: hash put: newList.
					newList := Array new: size * 2.
					newList
						replaceFrom: 1
						to: size
						with: valueList
						startingAt: 1.
					values at: hash put: newList ].
				tallies at: hash put: size.
				tally := tally - 1.
				^key ] ] ].
	^aBlock value! !

!LargeIdentityDictionary methodsFor: 'accessing' stamp: 'ul 3/24/2010 21:57'!
values

	^Array new: tally streamContents: [ :stream |
		self valuesDo: [ :value | stream nextPut: value ] ]! !

!LargeIdentityDictionary methodsFor: 'enumerating' stamp: 'ul 3/24/2010 21:46'!
valuesDo: aBlock

	tally = 0 ifTrue: [ ^self ].
	1 to: 4096 do: [ :arrayIndex |
		| valueArray |
		valueArray := values at: arrayIndex.
		1 to: (tallies at: arrayIndex) do: [ :index |
			aBlock value: (valueArray at: index) ] ]! !

!LargeIdentityHashedCollection classSide methodsFor: 'as yet unclassified' stamp: 'ul 12/18/2011 20:07'!
initialize

	| rng |
	rng := Random seed: 664399324.
	PermutationMap := (0 to: 4095) asArray shuffleBy: rng! !

!LargeIdentityHashedCollection classSide methodsFor: 'as yet unclassified' stamp: 'ul 12/18/2011 11:24'!
permuteHash: anInteger
	"Return an integer between 1 and 4096 when the argument is between 1 and 4096."

	^PermutationMap at: anInteger! !

!LargeIdentityHashedCollection methodsFor: 'initialize-release' stamp: 'ul 12/18/2011 11:31'!
initialize

	tally := 0.
	tallies := Array new: 4096 withAll: 0! !

!LargeIdentityHashedCollection methodsFor: 'as yet unclassified' stamp: 'ul 12/18/2011 11:35'!
postCopy

	tallies := tallies copy
! !

!LargeIdentityHashedCollection methodsFor: 'accessing' stamp: 'ul 12/18/2011 11:33'!
size

	^tally! !

LargeIdentityHashedCollection subclass: #LargeIdentitySet
	instanceVariableNames: 'array includesNil'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Collections'!

!LargeIdentitySet commentStamp: 'HenrikSperreJohansen 12/16/2011 12:55' prior: 0!
A LargeIdentitySet is an IdentitySet for large collections.
Rather than using linear probing, it takes advantage of the fact that identityHash (on a classic, 32bit VM) 
in Pharo only has 4096 unique values, using a bucket for each of those.
It will still work if hash range changes (ie buckets are chosen mod 4096), 
but the potential gain will be lower the more diverse the hash space is.

With linear probing you risk an array looking like this:
Index:	Hash:
	1	X
	2	X
	3	X
	4	4
	5	5
	6	4
	7	4
	8	7
	9	6
	10	X

While with buckets the same dataset looks:
Index:	Hash:
	1	X
	2	X
	3	X
	4	[4, 4, 4]
	5	5
	6	6
	7	7
	8	X
	9	X
	10	X

So includes: can generally be done faster (also sped up byusing a special primitive), 
and removal of objects does not have to do extensive cleanup if object was part of a chain.!

!LargeIdentitySet methodsFor: 'adding' stamp: 'ul 12/18/2011 11:22'!
add: anObject

	| hash |
	anObject ifNil: [ 
		includesNil ifFalse: [
			includesNil := true.
			tally := tally + 1 ].
		^anObject ].
	(array at: (hash := anObject largeIdentityHash + 1))
		ifNil: [ 
			array at: hash put: (Array with: anObject).
			tallies at: hash put: 1.
			tally := tally + 1 ]
		ifNotNil: [ :list | 
			(list pointsTo: anObject) ifFalse: [
				| newIndex |
				tally := tally + 1.
				list size < (newIndex := (tallies at: hash) + 1)
					ifFalse: [ list at: newIndex put: anObject ]
					ifTrue: [
						| newList |
						newList := Array new: list size * 2.
						newList
							replaceFrom: 1 
							to: list size
							with: list
							startingAt: 1;
							at: newIndex put: anObject.
						array at: hash put: newList ].
				tallies at: hash put: newIndex ] ].
	^anObject! !

!LargeIdentitySet methodsFor: 'adding' stamp: 'ul 12/18/2011 11:22'!
addIfNotPresent: anObject ifPresentDo: aBlock

	| hash |
	anObject ifNil: [ 
		includesNil ifFalse: [
			includesNil := true.
			tally := tally + 1 ].
		^anObject ].
	(array at: (hash := anObject largeIdentityHash + 1))
		ifNil: [ 
			array at: hash put: (Array with: anObject).
			tallies at: hash put: 1.
			tally := tally + 1 ]
		ifNotNil: [ :list | 
			(list pointsTo: anObject) 
			ifTrue: [ aBlock value]
			ifFalse: [
				| newIndex |
				tally := tally + 1.
				list size < (newIndex := (tallies at: hash) + 1)
					ifFalse: [ list at: newIndex put: anObject ]
					ifTrue: [
						| newList |
						newList := Array new: list size * 2.
						newList
							replaceFrom: 1 
							to: list size
							with: list
							startingAt: 1;
							at: newIndex put: anObject.
						array at: hash put: newList ].
				tallies at: hash put: newIndex ] ].
	^anObject! !

!LargeIdentitySet methodsFor: 'enumerating' stamp: 'ul 3/23/2010 08:35'!
do: aBlock

	tally = 0 ifTrue: [ ^self ].
	includesNil ifTrue: [ aBlock value: nil ].
	1 to: 4096 do: [ :arrayIndex |
		| subArray |
		subArray := array at: arrayIndex.
		1 to: (tallies at: arrayIndex) do: [ :index |
			aBlock value: (subArray at: index) ] ]! !

!LargeIdentitySet methodsFor: 'testing' stamp: 'ul 12/18/2011 11:22'!
includes: anObject

	anObject ifNil: [ ^includesNil ].
	^(array at: (anObject largeIdentityHash + 1))
		ifNil: [ false ]
		ifNotNil: [ :list | list pointsTo: anObject ]! !

!LargeIdentitySet methodsFor: 'initialize-release' stamp: 'ul 12/18/2011 11:31'!
initialize

	super initialize.
	array := Array new: 4096.
	includesNil := false! !

!LargeIdentitySet methodsFor: 'copying' stamp: 'ul 12/18/2011 11:35'!
postCopy

	super postCopy.
	array := array copy.
	1 to: 4096 do: [ :index |
		(array at: index) ifNotNil: [ :list |
			array at: index put: list copy ] ]! !

!LargeIdentitySet methodsFor: 'removing' stamp: 'ul 12/18/2011 11:22'!
remove: anObject ifAbsent: aBlock

	| list hash size |
	anObject ifNil: [
		includesNil ifTrue: [
			includesNil := false.
			tally := tally - 1.
			^anObject ].
		^aBlock value ].
	list := (array at: (hash := anObject largeIdentityHash + 1)) ifNil: [ ^aBlock value ].
	(list pointsTo: anObject) ifFalse: [ ^aBlock value ].
	1 to: (size := tallies at: hash) do: [ :index |
		(list at: index) == anObject 	ifTrue: [
			list
				at: index put: (list at: size);
				at: size put: nil.
			(size := size - 1) < (list size // 4) ifTrue: [
				| newList |
				newList := Array new: size * 2.
				newList
					replaceFrom: 1
					to: size
					with: list
					startingAt: 1.
				array at: hash put: newList ].
			tallies at: hash put: size.
			tally := tally - 1.
			^anObject ] ].
	^aBlock value! !

!SystemDictionary methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:36'!
fuelAccept: aVisitor

	^aVisitor visitWellKnownObject: self! !

!Boolean methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:36'!
fuelAccept: aVisitor

	^aVisitor visitWellKnownObject: self! !

!Rectangle methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 22:55'!
fuelAccept: aVisitor

	^ aVisitor visitRectangle: self
! !

!Quadrangle methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 22:55'!
fuelAccept: aVisitor

	^ aVisitor visitFixedObject: self
! !

!Trait methodsFor: '*Fuel' stamp: 'MartinDias 7/1/2011 04:32'!
basicEnvironment

	^self environment ! !

!Trait methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:33'!
fuelAccept: aVisitor

	^aVisitor visitTrait: self! !

!Float class methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 15:35'!
materializeFrom: aReadStream

	^ (self new: 2)
		at: 1 put: (aReadStream nextNumber: 4);
		at: 2 put: (aReadStream nextNumber: 4);
		yourself.
! !

!Float methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/6/2011 18:15'!
fuelAccept: aVisitor

	^aVisitor visitFloat: self! !

!Float methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 15:34'!
serializeOn: aWriteStream

	aWriteStream 
		nextNumber: 4 put: (self at: 1);
		nextNumber: 4 put: (self at: 2).! !

!Metaclass methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:26'!
fuelAccept: aVisitor

	^aVisitor visitMetaclass: self
! !

!ContextPart class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 5/22/2011 23:44'!
newFromFrameSize: aFrameSize
	
	^ super basicNew: aFrameSize! !

Stream subclass: #FLBufferedWriteStream
	instanceVariableNames: 'stream buffer position streamRespondsToNextPutAllStartingAt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Streams'!

!FLBufferedWriteStream commentStamp: 'MarianoMartinezPeck 6/5/2011 12:41' prior: 0!
FLBufferedWriteStream is a buffered write stream we use for Fuel serialization. Instead of directly using the stream provided to FLSerializer at creation time by the user, we create an instance of FLBufferedWriteStream for that stream.

MultiByteFileStream has no real buffer and goes to disk too frequently. With FLBufferedWriteStream we keep stuff in a cache and only go to disk when this is full.

The way of using it is jut FLBufferedWriteStream on: aWriteStream. For example:

FLBufferedWriteStream on: (FileDirectory default forceNewFileNamed:  'TestFile') binary

With the message #sizeBuffer: you can set the size of the buffer.

Make sure to always send #flush or #close when you're done, otherwise the last buffer might not yet have been written.
!

!FLBufferedWriteStream class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
on: writeStream
	^ self basicNew
		on: writeStream;
		yourself! !

!FLBufferedWriteStream class methodsFor: 'convenience' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
on: writeStream do: block
	"Execute block with as argument a ZnBufferedWriteStream on writeStream,
	making sure #flush is called at the end."
	
	| bufferedWriteStream |
	bufferedWriteStream := self on: writeStream.
	block value: bufferedWriteStream.
	bufferedWriteStream flush! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
buffer
	buffer isNil 
		ifTrue: [ self sizeBuffer: self defaultBufferSize ].
	^ buffer! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/23/2011 22:34'!
bufferFreeSize
	^ buffer size - position! !

!FLBufferedWriteStream methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
close
	self flushBuffer.
	stream close! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 9/8/2011 15:35'!
copyWordObjectToBuffer: aWordObject

	| blt |
	
	blt := (BitBlt current
				toForm: (Form new hackBits: self buffer))
				sourceForm: (Form new hackBits: aWordObject).
	blt combinationRule: Form over. "store"
	blt sourceX: 0;
		 sourceY: 0;
		 height: aWordObject byteSize // 4;
		 width: 4.
	blt destX: 0;
		 destY:  position // 4 .
	blt copyBits.

! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
defaultBufferSize
	^ 8192 "2 raisedTo: 13  "
	

! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
flush
	self flushBuffer.
	stream flush! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 10/22/2011 12:46'!
flushBuffer
	position = 0 ifTrue: [ ^ self ].
	position = buffer size
		ifTrue: [
			stream nextPutAll: buffer ]
		ifFalse: [
			streamRespondsToNextPutAllStartingAt
				ifTrue: [ stream next: position putAll: buffer startingAt: 1 ]
				ifFalse: [ stream nextPutAll: (buffer copyFrom: 1 to: position) ] ].
	position := 0! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 9/23/2011 22:35'!
flushBufferIfFull
	position = buffer size 
		ifTrue: [ self flushBuffer ]
! !

!FLBufferedWriteStream methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/22/2011 12:46'!
initializeStream: aWriteStream
	stream := aWriteStream.
	"This is ugly, but it is an optimization for #flushBuffer"
	streamRespondsToNextPutAllStartingAt := (stream respondsTo: #next:putAll:startingAt:).! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 10/6/2011 20:50'!
int16
	"Answer the next signed, 16-bit integer from this (binary) stream."

	| n |
	n := self next.
	n := (n bitShift: 8) + (self next).
	n >= 16r8000 ifTrue: [n := n - 16r10000].
	^ n
! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 9/9/2011 17:15'!
int32
	"Read a 32-bit signed integer from the next 4 bytes"
	| s |
	s := 0.
	1 to: 4 do: [:i | s := (s bitShift: 8) + self next].
	(s bitAnd: 16r80000000) = 0
		ifTrue: [^ s]
		ifFalse: [^ -1 - s bitInvert32]! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'mariano 10/6/2011 11:39'!
int32: int32
	"Write a signed integer to the next 4 bytes"
	| pos |
	pos := int32 < 0
		ifTrue: [(0-int32) bitInvert32 + 1]
		ifFalse: [int32].
	1 to: 4 do: [:i | self nextPut: (pos digitAt: 5-i)].
	^ int32! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 10/6/2011 20:51'!
int64
	"Answer the next signed, 32-bit integer from this (binary) stream."
	"Details: As a fast check for negative number, check the high bit of the first digit"

	| n firstDigit |
	n := firstDigit := self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	firstDigit >= 128 ifTrue: [n := -16r10000000000000000 + n].  "decode negative 64-bit integer"
	^ n
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 10/6/2011 20:35'!
int64: anInteger
	"Store the given signed, 64-bit integer on this (binary) stream."

	| n |
	(anInteger < -16r8000000000000000) | (anInteger >= 16r8000000000000000)
		ifTrue: [self error: 'outside 64-bit integer range'].

	anInteger < 0
		ifTrue: [n := 16r10000000000000000 + anInteger]
		ifFalse: [n := anInteger].
	self nextPut: (n digitAt: 8).
	self nextPut: (n digitAt: 7).
	self nextPut: (n digitAt: 6).
	self nextPut: (n digitAt: 5).
	self nextPut: (n digitAt: 4).
	self nextPut: (n digitAt: 3).
	self nextPut: (n digitAt: 2).
	self nextPut: (n digitAt: 1).
! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 10/22/2011 12:17'!
int8
	| val |
	val := self next.
	val >= 128 ifTrue: [^(256 - val) negated].
	^val! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 10/9/2011 22:47'!
nextBytesPutAll: collection
	self flushBufferIfFull.
	collection size <= self bufferFreeSize
		ifTrue: [
			self buffer replaceFrom: position + 1 to: position + collection size with: collection.
			position := position + collection size ]
		ifFalse: [
			self flushBuffer.
			collection size > (self buffer size / 2)
				ifTrue: [ stream nextBytesPutAll: collection ]
				ifFalse: [ self nextBytesPutAll: collection ] ]
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 10/6/2011 11:49'!
nextNumber: n put: v 
	"Append to the receiver the argument, v, which is a positive 
	SmallInteger or a LargePositiveInteger, as the next n bytes.
	Possibly pad with leading zeros."
	1 to: n do: [:i | self nextPut: (v digitAt: n+1-i)].
	^ v
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextPut: object
	self flushBufferIfFull.
	position := position + 1.
	self buffer at: position put: object! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 9/23/2011 22:35'!
nextPutAll: collection
	self flushBufferIfFull.
	collection size <= self bufferFreeSize
		ifTrue: [
			self buffer replaceFrom: position + 1 to: position + collection size with: collection.
			position := position + collection size ]
		ifFalse: [
			self flushBuffer.
			collection size > (self buffer size / 2)
				ifTrue: [ stream nextPutAll: collection ]
				ifFalse: [ self nextPutAll: collection ] ]
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 10/10/2011 12:33'!
nextStringPut: s 
	"Append the string, s, to the receiver.  Only used by DataStream.  Max size of 64*256*256*256."

	| length |
	(length := s size) < 192
		ifTrue: [self nextPut: length]
		ifFalse: 
			[self nextPut: (length digitAt: 4)+192.
			self nextPut: (length digitAt: 3).
			self nextPut: (length digitAt: 2).
			self nextPut: (length digitAt: 1)].
	self nextBytesPutAll: s .
	^s! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 9/23/2011 22:35'!
nextWordsPut: aWordObject
	| byteSize |

	byteSize := aWordObject byteSize.

	"Ensure we are at bigger than the words added, with size next power-of-two"
	byteSize > buffer size ifTrue: [ self sizeBuffer: 1 << ( byteSize highBit )].
	
	"BitBlt needs word-aligned access of object. Flushing the buffer is a very good idea because after the position will be zero, which is word aligned.  Word objects always have a full number of words of data to write. (otherwise they'd be variableByte objects or something :P) So as long as the size you write per instance also takes N words, the position will always be aligned (except when writing the first object of a cluster) . After that, we flush when an object larger than current buffer is encountered (statistically rare), or if the buffer is full (which we do anyways).
	Finally, we also need enough free space in the buffer"
	((position bitAnd: 3) = 0 and: [byteSize < self bufferFreeSize])
      	ifFalse: [self flushBuffer].

	self copyWordObjectToBuffer: aWordObject.
	position := position +  byteSize.! !

!FLBufferedWriteStream methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/22/2011 12:46'!
on: writeStream
	self initializeStream: writeStream.
	position := 0! !

!FLBufferedWriteStream methodsFor: 'printing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/8/2011 15:55'!
sizeBuffer: size
    buffer ifNotNil: [self flushBuffer].
    buffer := (stream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: size.
! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
space
	self nextPut: Character space! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MartinDias 9/27/2011 20:49'!
target
	^ stream! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'sfdas 9/23/2011 20:46'!
uint16
	"Answer the next unsigned, 16-bit integer from this (binary) stream."

	| n |
	n := self next.
	n := (n bitShift: 8) + (self next).
	^ n
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'mariano 10/6/2011 11:39'!
uint16: anInteger
	"Append to the receiver an Integer as the next two bytes."
	self nextPut: (anInteger bitShift: -8).
	self nextPut: (anInteger bitAnd: 255).
	^anInteger! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 9/9/2011 17:16'!
uint32
	"Read a 32-bit signed integer from the next 4 bytes"
	| s |
	s := 0.
	1 to: 4 do: [:i | s := (s bitShift: 8) + self next].
	^ s! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'mariano 10/6/2011 11:39'!
uint32: anInteger
	"Append to the receiver an Integer as the next 4 bytes."
	self nextPut: (anInteger bitShift: -24);
		nextPut: ((anInteger bitShift: -16) bitAnd: 255);
		nextPut: ((anInteger bitShift: -8) bitAnd: 255);
		nextPut: (anInteger bitAnd: 255).
	^ anInteger! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 10/6/2011 20:53'!
uint64
	"Answer the next unsigned, 64-bit integer from this (binary) stream."

	| n |
	n := self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	^ n
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 10/6/2011 20:31'!
uint64: anInteger
	"Store the given unsigned, 64-bit integer on this (binary) stream."

	(anInteger < 0) | (anInteger >= 16r10000000000000000)
		ifTrue: [self error: 'outside unsigned 64-bit integer range'].
		
	self nextPut: (anInteger digitAt: 8).
	self nextPut: (anInteger digitAt: 7).
	self nextPut: (anInteger digitAt: 6).
	self nextPut: (anInteger digitAt: 5).
	self nextPut: (anInteger digitAt: 4).
	self nextPut: (anInteger digitAt: 3).
	self nextPut: (anInteger digitAt: 2).
	self nextPut: (anInteger digitAt: 1).
! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 10/6/2011 20:52'!
uint8
	"Answer the next unsigned, 16-bit integer from this (binary) stream."

	^ self next.
! !

!Stream methodsFor: '*Fuel' stamp: 'MartinDias 8/13/2011 14:29'!
nextBytesPutAll: aCollection
	"Append the bytes of aCollection to the sequence of bytes accessible 
	by the receiver. Answer aCollection."

	^self nextPutAll: aCollection! !

!ClassTrait methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:23'!
fuelAccept: aVisitor

	^aVisitor visitClassTrait: self
! !

!Bitmap class methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 14:17'!
materializeFrom: aReadStream

	^self newFromStream: aReadStream! !

!Bitmap methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 17:45'!
fuelAccept: aVisitor

	^aVisitor visitBitmap: self! !

!Bitmap methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 14:17'!
serializeOn: aWriteStream

	self writeOn: aWriteStream! !

!Array methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 12/6/2011 00:29'!
fuelAccept: aVisitor
	
	"This IF is because there are subclasses of Array that are weak. If we do not put this IF, all subclasses fuel be using the #visitVariableObject:, loosing the capability of the weak
	However, this method is just to optimize the method lookup of the message #fuelAccept:. If this method is removed, the default behavior of Object shuld work as well. If the performance is not that different, then we can remove this method which has an ugly if."
	self class isWeak
		ifTrue: [ ^aVisitor visitWeakObject: self ]
		ifFalse: [ ^aVisitor visitVariableObject: self ]
	
	! !

!MethodContext methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:28'!
fuelAccept: aVisitor

	^aVisitor visitMethodContext: self! !

!MethodContext methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 5/22/2011 22:11'!
method: aMethod

	method := aMethod ! !

!Time class methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:30'!
materializeFrom: aReadStream
	
	^ self seconds: aReadStream int32 nanoSeconds: aReadStream int32! !

!Time methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 10:32'!
fuelAccept: aVisitor

	^ aVisitor visitTime: self
! !

!Time methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:29'!
serializeOn: aWriteStream

	aWriteStream 
		int32: self asSeconds;
		int32: self nanoSecond.! !

!CharacterBlock methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 22:56'!
fuelAccept: aVisitor

	^ aVisitor visitFixedObject: self
! !

!Duration class methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:26'!
materializeFrom: aReadStream

	^ self seconds: aReadStream int32 nanoSeconds: aReadStream int32 ! !

!Duration methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 10:31'!
fuelAccept: aVisitor

	^ aVisitor visitDuration: self
! !

!Duration methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:26'!
serializeOn: aWriteStream
	aWriteStream 
		int32: self asSeconds;
		int32: self nanoSeconds! !

!SmallInteger methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:31'!
fuelAccept: aVisitor

	^aVisitor visitSmallInteger: self! !

!Date class methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:09'!
materializeFrom: aReadStream
	^ self 
		starting: (DateAndTime julianDayNumber: aReadStream int32) midnight 
		duration: (Duration days: 1)! !

!Date methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 10:31'!
fuelAccept: aVisitor

	^ aVisitor visitDate: self
! !

!Date methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:07'!
serializeOn: aWriteStream

	aWriteStream 
		int32: self julianDayNumber! !

!HashedCollection methodsFor: '*Fuel' stamp: 'MartinDias 6/13/2011 02:43'!
fuelAfterMaterialization

	self rehash! !

!Point methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 21:42'!
fuelAccept: aVisitor

	^ aVisitor visitPoint: self
! !

!DateAndTime class methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 16:11'!
materializeFrom: aReadStream

	^ self basicNew
		fuelSet: aReadStream int32
		nanoSecond: aReadStream int32
		seconds: aReadStream int32
		offset: (Duration seconds: aReadStream int32 nanoSeconds: aReadStream int32)! !

!DateAndTime methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 15:17'!
fuelAccept: aVisitor

	^ aVisitor visitDateAndTime: self
! !

!DateAndTime methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 15:15'!
fuelSet: julianDayNumber nanoSecond: nanoSeconds seconds: numberOfSeconds offset: anOffset
	jdn := julianDayNumber.
	nanos := nanoSeconds.
	seconds := numberOfSeconds.
	offset := anOffset.! !

!DateAndTime methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 15:55'!
serializeOn: aWriteStream

	aWriteStream 
		int32: self julianDayNumber;
		int32: self nanoSecond;
		int32: self secondsSinceMidnight;
		int32: self offset asSeconds;
		int32: self offset nanoSeconds.! !

!LargePositiveInteger methodsFor: '*Fuel' stamp: 'MartinDias 12/13/2011 02:57'!
fuelAccept: aVisitor

	^aVisitor visitBytesObject: self! !

!ByteString class methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 01:31'!
materializeFrom: aReadStream

	^ aReadStream nextString! !

!ByteString methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 17:20'!
fuelAccept: aVisitor

	^aVisitor visitByteString: self! !

!ByteString methodsFor: '*Fuel' stamp: 'MartinDias 12/12/2011 01:30'!
serializeOn: aWriteStream

	aWriteStream nextStringPut: self! !

!MethodDictionary methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/20/2011 09:41'!
fuelAfterMaterialization

	self fuelRehashWithoutBecome! !

!MethodDictionary methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/27/2011 20:52'!
fuelRehashWithoutBecome
	"The original #rehash method in Pharo 1.0 to 1.3 is terrible slow because it uses a #become. Since Pharo 1.4 14155 there is a new version that avoids the #become and uses instead a #copyFrom, which is MUCH faster. The following code is exactly that new version of #rehash. We put it here in order to be compatible with previous versions of Pharo. In the future, it must be removed and directly use #rehash instead."
	  | newInstance |
	  newInstance := (self species basicNew: self basicSize) initialize: self basicSize.
        1 to: self basicSize do: [ :index | 
                (self basicAt: index) ifNotNil: [ :key |
                        newInstance at: key put: (array at: index) ] ].
        self copyFrom: newInstance! !

!TPureBehavior methodsFor: '*Fuel' stamp: 'MartinDias 8/29/2011 04:06'!
fuelInitializeTraitComposition

	self traitComposition traits do: [:each | each addUser: self]! !

!ClassDescription methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 19:23'!
basicInstVarNames: anArray

	instanceVariables := anArray! !

!ClassDescription methodsFor: '*Fuel' stamp: 'MartinDias 7/27/2011 22:33'!
fuelInstVarNamesAndOffsetsDo: aBinaryBlock
	"This is part of the interface between the compiler and a class's instance or field names.
	 The class should enumerate aBinaryBlock with the instance variable name strings and
	 their integer offsets.  The order is important. Names evaluated later will override the
	 same names occurring earlier."

	| superInstSize |
	(superInstSize := superclass notNil ifTrue: [superclass instSize] ifFalse: [0]) > 0 ifTrue:
		[superclass fuelInstVarNamesAndOffsetsDo: aBinaryBlock].
	1 to: self instSize - superInstSize do:
		[:i| aBinaryBlock value: (instanceVariables at: i) value: i + superInstSize]! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 18:23'!
basicClassPool
	"Answer nil or the dictionary of class variables."

	^classPool! !

!Class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 6/29/2011 11:04'!
basicEnvironment

	^ environment! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 18:23'!
basicSharedPools
	"Answer nil or a Set of the pool dictionaries declared in the receiver."

	^sharedPools! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 5/14/2011 21:44'!
basicSubclasses

	^ subclasses! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 5/14/2011 21:45'!
basicSubclasses: anArray

	subclasses := anArray! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:22'!
fuelAccept: aVisitor

	^aVisitor visitClass: self! !

Object subclass: #FLAnalysis
	instanceVariableNames: 'objectStack firstMapper clusterization root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLAnalysis commentStamp: 'MartinDias 8/29/2011 19:15' prior: 0!
I am responsible of traversing the graph of references starting from a root object. I will produce a clusterization which the serializer will store on a stream.

An example of use is:

	(FLAnalyzer newDefault 
		clusterizationOf: (Array with: 1@2 with: 3@4))
		clusters.
!

!FLAnalysis class methodsFor: 'instance creation' stamp: 'MartinDias 9/9/2011 22:13'!
newWith: aMapper root: anObject

	^self basicNew
		initializeWith: aMapper root: anObject;
		yourself.! !

!FLAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 8/7/2011 20:18'!
clusterFor: anObject

	^firstMapper chainedClusterFor: anObject! !

!FLAnalysis methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 21:47'!
clusterization

	^clusterization! !

!FLAnalysis methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:43'!
fuelAccept: aVisitor  
	^ aVisitor visitSubstitution: self! !

!FLAnalysis methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:42'!
fuelSubstitution 
	^ nil! !

!FLAnalysis methodsFor: 'initialize-release' stamp: 'MartinDias 12/4/2011 02:35'!
initializeWith: aMapper root: anObject

	self initialize.
	firstMapper := aMapper.
	root := anObject.
	objectStack := FLSimpleStack new.
	clusterization := FLClusterization new.
! !

!FLAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:22'!
mapAndTrace: anObject
	"Map an object to its cluster. Trace its references."

	| aCluster |
	aCluster := self clusterFor: anObject.
	clusterization add: aCluster traceWith: self.
	aCluster add: anObject traceWith: self.
! !

!FLAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 9/10/2011 18:08'!
run

	objectStack push: root.
	[ objectStack isEmpty ] 
		whileFalse: [ self mapAndTrace: objectStack pop ].! !

!FLAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:11'!
trace: anObject

	objectStack push: anObject! !

Object subclass: #FLAnalyzer
	instanceVariableNames: 'internalClasses pluggableSubstitutions globalMappersFactory analysisFactory generalMapperFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLAnalyzer commentStamp: 'MartinDias 8/29/2011 19:15' prior: 0!
I am responsible of traversing the graph of references starting from a root object. I will produce a clusterization which the serializer will store on a stream.

An example of use is:

	(FLAnalyzer newDefault 
		clusterizationOf: (Array with: 1@2 with: 3@4))
		clusters.
!

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 9/9/2011 21:25'!
newDefault
	"Returns an instance with default configuration."

	^self new! !

!FLAnalyzer methodsFor: 'analyzing' stamp: 'MartinDias 9/14/2011 00:15'!
analysisFor: anObject

	^analysisFactory value: anObject! !

!FLAnalyzer methodsFor: 'configuring' stamp: 'MarianoMartinezPeck 12/14/2011 20:42'!
considerGlobalMinimaly

	globalMappersFactory := [self minimalGlobalsMappers].
	generalMapperFactory := [FLLightGeneralMapper new].! !

!FLAnalyzer methodsFor: 'analyzing' stamp: 'MartinDias 9/10/2011 18:10'!
firstInMapperChain

	^self mappers
		reduceRight: [:left :right | left next: right ];
		first
! !

!FLAnalyzer methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:43'!
fuelAccept: aVisitor  
	^ aVisitor visitSubstitution: self! !

!FLAnalyzer methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:43'!
fuelSubstitution 
	^ nil! !

!FLAnalyzer methodsFor: 'defaults' stamp: 'MarianoMartinezPeck 12/14/2011 20:43'!
generalMapper

	^generalMapperFactory value! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/11/2011 11:22'!
globalMappers
	
	^globalMappersFactory value! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 9/13/2011 19:25'!
initialize

	super initialize.
	self setDefaultAnalysis.
	self considerGlobalMinimaly.
	pluggableSubstitutions := OrderedCollection new.
	internalClasses := OrderedCollection new.
! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MarianoMartinezPeck 12/14/2011 20:43'!
mappers

	| mappers |
	mappers := OrderedCollection new.
	mappers addAll: self pluggableSubstitutionMappers.
	mappers addAll: self globalMappers.
	mappers add: self generalMapper.
	^mappers! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/11/2011 12:33'!
minimalGlobalsMappers

	^Array with: FLLightGlobalMapper new! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/10/2011 17:55'!
pluggableSubstitutionMappers

	^pluggableSubstitutions collect: [:aLink |
		FLPluggableSubstitutionMapper 
			when: aLink key
			substituteBy: aLink value]! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/14/2011 00:24'!
setDefaultAnalysis

	analysisFactory := [:anObject | 
		(FLAnalysis 
			newWith: self firstInMapperChain 
			root: anObject)
			run;
			yourself ]! !

!FLAnalyzer methodsFor: 'configuring' stamp: 'MartinDias 9/10/2011 17:04'!
when: aCondition substituteBy: aFactory

	pluggableSubstitutions add: aCondition -> aFactory! !

Object subclass: #FLCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCluster commentStamp: 'MartinDias 8/29/2011 19:20' prior: 0!
I represent a cluster of objects grouped by some specific similarity. I know how to serialize and materialize them all together.!

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 10/13/2011 16:26'!
add: anObject traceWith: aAnalysis
	"Add an object to the cluster and trace references."

	self subclassResponsibility! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
afterMaterializationStepWith: aMaterialization
! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:54'!
clusterBucketIn: aClusterization
	"Answer in which clusters bucket place this cluster. This defines the sort order in which the clusters will be ordered. See FLAnalyzer >> clusteredObjects for more information."

	self subclassResponsibility ! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:54'!
clusterReferencesDo: aBlock
	"Evaluate a block with each object referenced by the cluster"! !

!FLCluster methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:47'!
fuelAccept: aVisitor  
	^ aVisitor visitSubstitution: self! !

!FLCluster methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:47'!
fuelSubstitution 
	^ nil! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstancesStepWith: aMaterialization

	self subclassResponsibility ! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializePostInstancesStepWith: aMaterialization

! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeReferencesStepWith: aMaterialization
! !

!FLCluster methodsFor: 'accessing' stamp: 'MartinDias 8/29/2011 00:50'!
objects
	"Answer the objects that this cluster groups."

	self subclassResponsibility! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 01:23'!
registerIndexesOn: aDictionary
	"Maps objects with indexes in the dictionary."! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstancesStepWith: aSerialization

	self subclassResponsibility ! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializePostInstancesStepWith: aSerialization
! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeReferencesStepWith: aSerialization
! !

FLCluster subclass: #FLIteratingCluster
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLIteratingCluster commentStamp: '<historical>' prior: 0!
I am a template class whose algorithm for serialize a collection of objects consists on delegate the serialization of each individual object to the subclass.!

FLIteratingCluster subclass: #FLBlockClosureCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLBlockClosureCluster commentStamp: 'MarianoMartinezPeck 12/3/2011 01:22' prior: 0!
FLBlockClosureCluster is a special cluster to optimize the way BlockClosure are serialized. 
A clean closure is one that doesn't really need the home context because:
		- It doesn't send messages to self or super
		- It doesn't access any instance variable
		- It doesn't access any outer temp
		- It doesn't do ^ return"
For more details read BlockClosure >> isClean.		
		
For clean closures it is not necessary to serialize the whole stack of MethodContexts, since they are not needed. However, the receiver and the method are stored in the outerContext of the BlockClosure. So, if the closure is clean we just serialize the closure itself, but instead of fully serializing the outerContext, we just serialize its method and receiver. During materialization, we materialize the closure and we create an empty MethodContext (nil sender and emtpy arguments) with just the receiver and the method. 

If the closure is not clean, then the whole outerContext (included its sender) is serialized, meaning the whole stack will be serialized. 

For more details read this mail: http://forum.world.st/Prune-stack-serialization-td4148300.html!

!FLBlockClosureCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 12/2/2011 23:54'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket! !

!FLBlockClosureCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/3/2011 00:33'!
materializeInstanceWith: aMaterialization

	^ BlockClosure basicNew: aMaterialization nextEncodedPositiveInteger! !

!FLBlockClosureCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/9/2011 23:49'!
materializeReferencesOf: aBlockClosure with: aMaterialization

	| methodOrContext method context |	
	1 to: aBlockClosure basicSize do: [ :index |
		aBlockClosure basicAt: index put: (aMaterialization nextEncodedReference).
	].

	(aMaterialization stream next = 1)
		ifTrue: [ " it was a clean BlockClosure, otherwise it should have been 0"
			method := aMaterialization nextEncodedReference.
			aBlockClosure 
					outerContext: 	(MethodContext
										sender: nil 
										receiver: aMaterialization nextEncodedReference 
										method: method 
										arguments: #() )
					startpc: aMaterialization nextEncodedReference 
					numArgs: aMaterialization nextEncodedReference.			
			]
		ifFalse: [	
				context := aMaterialization nextEncodedReference.
				aBlockClosure 
					outerContext: context
					startpc: aMaterialization nextEncodedReference 
					numArgs: aMaterialization nextEncodedReference].
	! !

!FLBlockClosureCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/2/2011 23:54'!
materializeReferencesStepWith: aMaterialization

	objects do: [ :anObject | self materializeReferencesOf: anObject with: aMaterialization ]! !

!FLBlockClosureCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 12/3/2011 01:08'!
referencesOf: aBlockClosure do: aBlock

	1 to: aBlockClosure basicSize do: [ :index |
		aBlock value: (aBlockClosure basicAt: index) ].

	aBlockClosure isClean
		ifTrue: [
			aBlock value: aBlockClosure method.
			aBlock value: aBlockClosure receiver.
			]
		ifFalse: [ aBlock value: aBlockClosure outerContext ].

	aBlock value: aBlockClosure startpc.
	aBlock value: aBlockClosure numArgs.
	
! !

!FLBlockClosureCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/3/2011 00:33'!
serializeInstance: aBlockClosure with: aSerialization

	aSerialization encodePositiveInteger: aBlockClosure basicSize! !

!FLBlockClosureCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/10/2011 00:00'!
serializeReferencesOf: aBlockClosure with: aSerialization	

	1 to: aBlockClosure basicSize do: [ :index |
		aSerialization encodeReferenceTo: (aBlockClosure basicAt: index) ].

	
	aBlockClosure isClean
		ifTrue: [
			aSerialization stream nextPut: 1.
			aSerialization encodeReferenceTo:  aBlockClosure method.
			aSerialization encodeReferenceTo:  aBlockClosure receiver.
			]
		ifFalse: [ 
			aSerialization stream nextPut: 0.
			aSerialization encodeReferenceTo:  aBlockClosure outerContext. 
		].
	
	aSerialization encodeReferenceTo:  aBlockClosure startpc.
	aSerialization encodeReferenceTo:  aBlockClosure numArgs.
	
! !

!FLBlockClosureCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/3/2011 00:32'!
serializeReferencesStepWith: aSerialization

	objects do: [ :anObject | self serializeReferencesOf: anObject with: aSerialization ]! !

FLIteratingCluster subclass: #FLCompiledMethodCluster
	instanceVariableNames: 'trailerEncoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCompiledMethodCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for CompiledMethod instances.!

!FLCompiledMethodCluster class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 9/23/2011 16:39'!
withNoSource
	^ self new initializeWithEncoder: FLCompiledMethodTrailerEncoderNoSource new.! !

!FLCompiledMethodCluster class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 9/23/2011 16:38'!
withNoTrailerChange
	^ self new initializeWithEncoder: FLCompiledMethodTrailerEncoderNoChange new.! !

!FLCompiledMethodCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/17/2011 19:31'!
clusterBucketIn: aClusterization

	^aClusterization primitivesBucket! !

!FLCompiledMethodCluster methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 9/23/2011 16:12'!
initializeWithEncoder: aTrailerEncoder
	trailerEncoder := aTrailerEncoder.! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	| compiledMethod header bytecodesPlusTrailerSize aReadStream |

	aReadStream := aMaterialization stream.

	header := aReadStream uint32.
	bytecodesPlusTrailerSize := aReadStream uint16.

	compiledMethod := CompiledMethod newMethod: bytecodesPlusTrailerSize header: header.

	compiledMethod initialPC to: compiledMethod size
		do: [ :index | compiledMethod at: index put: aReadStream next ].
		
	^ compiledMethod! !

!FLCompiledMethodCluster methodsFor: 'protected' stamp: 'MartinDias 10/13/2011 16:23'!
materializeLiteralsTo: compiledMethod with: aMaterialization

	1 to: compiledMethod numLiterals 
		do: [ :index | compiledMethod literalAt: index put: aMaterialization nextEncodedReference ]
! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeReferencesStepWith: aMaterialization

	objects do: [ :aCompiledMethod | self materializeLiteralsTo: aCompiledMethod with: aMaterialization ]! !

!FLCompiledMethodCluster methodsFor: 'analyzing' stamp: 'MartinDias 3/26/2011 16:27'!
referencesOf: anObject do: aBlock

	1 to: anObject numLiterals
		do: [ :index | aBlock value: (anObject literalAt: index) ]! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: aCompiledMethodToSerialize with: aSerialization

	| header aWriteStream bytecodesPlusTrailerSize cmSize cmInitialPC aCompiledMethod |

	aWriteStream := aSerialization stream.

	aCompiledMethod := trailerEncoder compiledMethodToSerialize: aCompiledMethodToSerialize.

	header := aCompiledMethod header.
	cmSize := aCompiledMethod size.
	cmInitialPC := aCompiledMethod initialPC.
	bytecodesPlusTrailerSize := cmSize - cmInitialPC + 1.

	aWriteStream 
		uint32: header;
		uint16: bytecodesPlusTrailerSize.	

	cmInitialPC to: cmSize
		do: [ :index | aWriteStream nextPut: (aCompiledMethod at: index) ].! !

!FLCompiledMethodCluster methodsFor: 'protected' stamp: 'MartinDias 10/13/2011 16:20'!
serializeLiteralsOf: aCompiledMethod with: aSerialization

	1 to: aCompiledMethod numLiterals 
		do: [ :index | aSerialization encodeReferenceTo: (aCompiledMethod literalAt: index) ]! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeReferencesStepWith: aSerialization

	objects do: [ :aCompiledMethod | self serializeLiteralsOf: aCompiledMethod with: aSerialization ]! !

FLCompiledMethodCluster subclass: #FLNamedPrimitiveMethodCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLNamedPrimitiveMethodCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for CompiledMethods which are <primitive:module:> primitives.

For example, Integer >> bitAnd: should be serialized with me.!

!FLNamedPrimitiveMethodCluster methodsFor: 'protected' stamp: 'MartinDias 10/13/2011 16:23'!
materializeLiteralsTo: compiledMethod with: aMaterialization

	| module name |

	module := aMaterialization nextEncodedReference.
	name := aMaterialization nextEncodedReference.
	"The third element of the array is Obsolete. We can put whatever we want since it is not used anymore by the VM. It used to be a session ID. The forth element is the primitive index in a externalPrimitiveTable and it is IMPORTANT to put it in zero. Always. Otherwise there can be errors. Putting a zero means that the VM will not try to directly execute the primitive from the table and it will always try to load the module. It is the safer thing to do. For more details, read the method #primitiveExternalCall in VMMaker"
	compiledMethod literalAt: 1 put: (Array with: module with: name with: 0 with: 0).

	2 to: compiledMethod numLiterals 
		do: [ :index | compiledMethod literalAt: index put: aMaterialization nextEncodedReference ]
! !

!FLNamedPrimitiveMethodCluster methodsFor: 'analyzing' stamp: 'MartinDias 11/11/2010 18:31'!
referencesOf: anObject do: aBlock

	aBlock 
		value: (anObject literalAt: 1) first;
		value: (anObject literalAt: 1) second. 

	2 to: anObject numLiterals
		do: [ :index | aBlock value: (anObject literalAt: index) ]! !

!FLNamedPrimitiveMethodCluster methodsFor: 'protected' stamp: 'MartinDias 10/13/2011 16:20'!
serializeLiteralsOf: aCompiledMethod with: aSerialization

	| primitiveArguments |
	
	primitiveArguments := aCompiledMethod literalAt: 1.
	aSerialization 
		encodeReferenceTo: primitiveArguments first;
		encodeReferenceTo: primitiveArguments second.

	2 to: aCompiledMethod numLiterals 
		do: [ :index | aSerialization encodeReferenceTo: (aCompiledMethod literalAt: index) ]! !

!FLIteratingCluster methodsFor: 'analyzing' stamp: 'MartinDias 10/13/2011 16:26'!
add: anObject traceWith: aAnalysis
	"Add an object to the cluster and trace references."

	objects addIfNotPresent: anObject ifPresentDo: [ ^self ].

	self referencesOf: anObject do: [ :aChild | aAnalysis trace: aChild ].! !

!FLIteratingCluster methodsFor: 'initialize-release' stamp: 'MartinDias 8/29/2011 00:47'!
initialize

	super initialize.
	objects := self newSerializingCollection.! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:22'!
materializeInstanceWith: aMaterialization
	"Hook method that each subclass should customize for its special way of materializing"

	^ self subclassResponsibility! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:22'!
materializeInstancesStepWith: aMaterialization

	| instanceCount |
	instanceCount := aMaterialization nextEncodedPositiveInteger.
	
	objects := self newMaterializingCollection: instanceCount.  
	
	instanceCount timesRepeat: [ 
		objects add: (self materializeInstanceWith: aMaterialization) ].! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/15/2011 12:36'!
newMaterializingCollection: instanceCount

	^(OrderedCollection new: instanceCount)
		resetTo: 1; "This avoids prematures #makeRoomAtLast. In fact, this has already been fixed in Pharo 1.4: http://code.google.com/p/pharo/issues/detail?id=1686"
		yourself.
! !

!FLIteratingCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 12/18/2011 22:53'!
newSerializingCollection
	"Answer a collection for the objects that correspond to this cluster."

	^ LargeIdentitySet new! !

!FLIteratingCluster methodsFor: 'accessing' stamp: 'MartinDias 8/29/2011 00:59'!
objects

	^objects! !

!FLIteratingCluster methodsFor: 'printing' stamp: 'MartinDias 8/29/2011 01:16'!
printNameOn: aStream

	super printOn: aStream! !

!FLIteratingCluster methodsFor: 'printing' stamp: 'MartinDias 8/29/2011 00:52'!
printOn: aStream

	self printNameOn: aStream.
	aStream nextPutAll: '->'.
	objects printElementsOn: aStream! !

!FLIteratingCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:56'!
referencesOf: anObject do: aBlock
	"Evaluate a block with each object referenced by anObject"! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 01:22'!
registerIndexesOn: aDictionary

	self objects do: [ :instance | aDictionary at: instance put: aDictionary size + 1 ].! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: anObject with: aSerialization
	"Hook method that each subclass should customize for its special way of serializing"

	self subclassResponsibility ! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstances: aSerialization
	objects do: [ :instance | self serializeInstance: instance with: aSerialization ]! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstancesStepWith: aSerialization
	aSerialization encodePositiveInteger: objects size.
	self serializeInstances: aSerialization! !

FLIteratingCluster subclass: #FLObjectCluster
	instanceVariableNames: 'theClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLObjectCluster commentStamp: 'MartinDias 8/29/2011 19:20' prior: 0!
I implement a generic way of serializing and materializing an object, for those that do not need more special way of doing it.!

FLObjectCluster subclass: #FLBitsObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLBitsObjectCluster commentStamp: 'MartinDias 8/1/2011 02:59' prior: 0!
I have the common behavior for generic storing and loading variable bits objects.!

!FLBitsObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 10/13/2011 16:26'!
add: anObject traceWith: aAnalysis
	"Add an object to the cluster. We know the object doesn't have references."

	objects addLast: anObject! !

!FLBitsObjectCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/27/2011 11:06'!
newSerializingCollection
	"Answer a collection for the objects that correspond to this cluster."

	^OrderedCollection new! !

!FLBitsObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:26'!
registerIndexesOn: aDictionary

	self flag: #todo. "Converting objects collection here in this method is a bit confusing. This is because since this cluster is for primitives, they do not have pointers to other objects. Hence, instead of storing them in a IdentitySet we can use an OrderedCollection and then just convert them at the end. For more details see FLBitsObjectCluster >> #add: anObject traceWith: aAnalysis"
	objects := objects asIdentitySet.
	super registerIndexesOn: aDictionary.! !

FLBitsObjectCluster subclass: #FLByteObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLByteObjectCluster commentStamp: 'MartinDias 8/1/2011 02:59' prior: 0!
I am the generic cluster for storing and loading variable byte objects.!

!FLByteObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	| instance size |
	size := aMaterialization nextEncodedPositiveInteger.
	instance := theClass basicNew: size.
	aMaterialization stream next: size into: instance.
	^ instance
! !

!FLByteObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: anObject with: aSerialization

	aSerialization encodePositiveInteger: anObject basicSize.
	aSerialization stream nextBytesPutAll: anObject.! !

FLBitsObjectCluster subclass: #FLWordObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLWordObjectCluster commentStamp: 'MarianoMartinezPeck 9/6/2011 22:50' prior: 0!
I am the generic cluster for storing and loading variable word objects. Most word-like objects are subclasses from ArrayedCollection. Hence, we may have used the method #writeOn: to serialize, and #newFromStream: to materialize. 

#writeOn: is slow because it ALWAYS encode wors in a big endian format. Most machines today are even little endian. Even worst, #newFromStream: has to always do a #restoreEndianness. 

#newFromStream: is fast because it uses the method #nextWordsInto: which is a hack but that is really fast. The problem is that #nextWordsInto: has to always do the #restoreEndianness when we are in little endian machines (most of the time).

Hence, this cluster provives the best of both worlds: the serialization is done with a platform-dependen way, that is, the endianness will be the same of the machine where we are serializing. In addition, we serialize with the same hack that #nextWordsInto: uses, that is, the Bitmap >> hackBits:
And the materialization will use a variation of #nextWordsInto:  (#fuelNextWordsInto:) that ONLY does the #restoreEndianness if the machine where we are materializing has a different endianness than where we have serialized the object. To do this, while serializing, we store in the header of the Fuel stream, the endianness. Then in materialization, we compare agains that. 

Conclusion: we only restore endianness when needed.

Notice, in addition, that the method #newFromStream: sends the message #restoreEndianness. This method EXPECTS that the object was serialized in big endian format, and it will only change the endianness if we are materializing in a little endian machine. This means that if you serialize in little endian and materialize in big endian, you are screw.  For this reason, this cluster uses a method that always change the endianness, Bitmap class >> swapBytesIn:from:to: 

Future work: maybe we can modify and use the Bitmap hackBits:   not only for materialization but also for serialization. 
!

!FLWordObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	| inst wideSize |

	wideSize :=  aMaterialization nextEncodedPositiveInteger.
	inst := theClass basicNew: wideSize.	

	aMaterialization stream fuelNextWordsInto: inst. 

	(aMaterialization isBigEndian = Smalltalk isBigEndian asBit)
		ifTrue: [^ inst ]
		ifFalse: [ ^ self swapBytesOf: inst ] 
	! !

!FLWordObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: aWordObject with: aSerialization

	aSerialization encodePositiveInteger: aWordObject basicSize.
	aSerialization stream nextWordsPut: aWordObject

! !

!FLWordObjectCluster methodsFor: 'endianness' stamp: 'MarianoMartinezPeck 9/6/2011 12:34'!
swapBytesOf: aWordObject
	Bitmap
                swapBytesIn: aWordObject
                from: 1
                to: aWordObject basicSize.
	^ aWordObject! !

FLObjectCluster subclass: #FLHookPrimitiveCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLHookPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 12/12/2011 01:25'!
add: anObject traceWith: aAnalysis
	"Add an object to the cluster. We know the object doesn't have references."

	objects addLast: anObject! !

!FLHookPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 12/12/2011 01:25'!
clusterBucketIn: aClusterization

	^aClusterization primitivesBucket ! !

!FLHookPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/12/2011 01:38'!
materializeInstanceWith: aMaterialization
	
	^theClass materializeFrom: aMaterialization stream! !

!FLHookPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 12/12/2011 01:25'!
newSerializingCollection
	"Answer a collection for the objects that correspond to this cluster."

	^OrderedCollection new! !

!FLHookPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/12/2011 01:25'!
registerIndexesOn: aDictionary

	self flag: #todo. "Converting objects collection here in this method is a bit confusing. This is because since this cluster is for primitives, they do not have pointers to other objects. Hence, instead of storing them in a IdentitySet we can use an OrderedCollection and then just convert them at the end. For more details see FLPrimitiveCluster >> #add: anObject traceWith: aAnalysis"
	objects := objects asIdentitySet.
	super registerIndexesOn: aDictionary.! !

!FLHookPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/12/2011 01:32'!
serializeInstance: anObject with: aSerialization

	anObject serializeOn: aSerialization stream! !

!FLObjectCluster class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 10/24/2011 16:40'!
for: aClass

	^ self basicNew
		initializeWith:aClass;
		yourself.! !

!FLObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 18:09'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket ! !

!FLObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 5/30/2011 03:45'!
clusterReferencesDo: aBlock

	aBlock value: theClass! !

!FLObjectCluster methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/24/2011 16:42'!
initializeWith: aClass
	super initialize.
	theClass := aClass! !

!FLObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstancesStepWith: aMaterialization

	theClass := aMaterialization nextEncodedReference.
	super materializeInstancesStepWith: aMaterialization.! !

!FLObjectCluster methodsFor: 'printing' stamp: 'MarianoMartinezPeck 10/24/2011 10:58'!
printNameOn: aStream
	"Append to the argument, aStream, a sequence of characters that  
	identifies the receiver."
	
	super printNameOn: aStream.
	aStream nextPut: $[.
	theClass printOn: aStream.
	aStream nextPut: $].
	! !

!FLObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstancesStepWith: aSerialization

	aSerialization encodeReferenceTo: theClass.
	super serializeInstancesStepWith: aSerialization.! !

!FLObjectCluster methodsFor: 'printing' stamp: 'MarianoMartinezPeck 12/9/2011 20:26'!
theClass 
	^ theClass! !

FLObjectCluster subclass: #FLPointerObjectCluster
	instanceVariableNames: 'variablesMapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPointerObjectCluster commentStamp: 'MartinDias 8/1/2011 03:00' prior: 0!
I have the common behavior for storing and loading pointer objects.!

FLPointerObjectCluster subclass: #FLFixedObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLFixedObjectCluster commentStamp: 'MartinDias 5/30/2011 01:25' prior: 0!
I am a generic cluster for objects without indexable variables.!

!FLFixedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	^theClass basicNew! !

!FLFixedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstances: aSerialization! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
afterMaterializationStepWith: aMaterialization  

	objects do: [ :anObject | anObject fuelAfterMaterialization ]! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstancesStepWith: aMaterialization
	"Template method that let to the subclass the materialization using the received materializer"

	super materializeInstancesStepWith: aMaterialization.
		
	variablesMapping := FLVariablesMapping for: theClass.
	variablesMapping materializeMappingFrom: aMaterialization stream.! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeReferencesOf: anObject with: aMaterialization

	variablesMapping materializeReferencesOf: anObject with: aMaterialization! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeReferencesStepWith: aMaterialization

	objects do: [ :anObject | self materializeReferencesOf: anObject with: aMaterialization ]! !

!FLPointerObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/27/2011 22:32'!
referencesOf: anObject do: aBlock

	| ignoredInstanceVariableNames |
	ignoredInstanceVariableNames := theClass fuelIgnoredInstanceVariableNames.
	
	theClass fuelInstVarNamesAndOffsetsDo: [:name :index | 
		(ignoredInstanceVariableNames includes: name) 
			ifFalse: [ aBlock value: (anObject instVarAt: index) ]]
! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstancesStepWith: aSerialization

	super serializeInstancesStepWith: aSerialization.
	
	variablesMapping := FLVariablesMapping for: theClass.
	variablesMapping serializeMappingOn: aSerialization stream.! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeReferencesOf: anObject with: aSerialization
	
	variablesMapping serializeReferencesOf: anObject with: aSerialization
! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeReferencesStepWith: aSerialization

	objects do: [ :anObject | self serializeReferencesOf: anObject with: aSerialization ]! !

FLPointerObjectCluster subclass: #FLVariableObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLVariableObjectCluster commentStamp: 'MartinDias 5/30/2011 01:25' prior: 0!
I am a generic cluster for objects with indexable variables.!

FLVariableObjectCluster subclass: #FLMethodContextCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLMethodContextCluster commentStamp: 'MartinDias 5/30/2011 01:18' prior: 0!
I am a cluster for MethodContexts.!

!FLMethodContextCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	^ theClass newFromFrameSize: aMaterialization stream next
! !

!FLMethodContextCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: aMethodContext with: aSerialization

	aSerialization stream nextPut: aMethodContext method frameSize.
! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	^theClass basicNew: aMaterialization nextEncodedPositiveInteger! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeReferencesOf: anObject with: aMaterialization

	super materializeReferencesOf: anObject with: aMaterialization.

	1 to: anObject basicSize 
		do: [ :index | anObject basicAt: index put: aMaterialization nextEncodedReference ]! !

!FLVariableObjectCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 12/6/2011 18:41'!
referencesOf: anObject do: aBlock

	super referencesOf: anObject do: aBlock.
	aBlock value: anObject basicSize.
	self variablePartReferencesOf: anObject do: aBlock. 

! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: anObject with: aSerialization

	aSerialization encodePositiveInteger: anObject basicSize! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/6/2011 19:05'!
serializeReferencesOf: anObject with: aSerialization

	super serializeReferencesOf: anObject with: aSerialization.
	self serializeReferencesVariablePartOf: anObject with: aSerialization. ! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/6/2011 19:05'!
serializeReferencesVariablePartOf: anObject with: aSerialization
	
	1 to: anObject basicSize do: [ :index |
		aSerialization encodeReferenceTo: (anObject basicAt: index) ]! !

!FLVariableObjectCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 12/6/2011 18:41'!
variablePartReferencesOf: anObject do: aBlock 

	1 to: anObject basicSize 
		do: [ :index | aBlock value: (anObject basicAt: index) ]! !

FLVariableObjectCluster subclass: #FLWeakVariableObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLWeakVariableObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 12/6/2011 03:16'!
clusterReferencesDo: aBlock

	super clusterReferencesDo: aBlock.
	aBlock value: nil! !

!FLWeakVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 12/6/2011 19:06'!
serializeReferencesVariablePartOf: anObject with: aSerialization
	
	1 to: anObject basicSize do: [ :index |
		aSerialization encodeWeakReferenceTo: (anObject basicAt: index) ]! !

!FLWeakVariableObjectCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 12/6/2011 19:22'!
variablePartReferencesOf: anObject do: aBlock 
	"Since the variable part of Weak classes are weak references, we should not do nothing here."! !

FLIteratingCluster subclass: #FLOptimizedObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLOptimizedObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 12/13/2011 02:59'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket ! !

!FLOptimizedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 10/22/2011 20:22'!
materializeReferencesStepWith: aMaterialization

	objects do: [ :anObject | self materializeReferencesOf: anObject with: aMaterialization ]! !

!FLOptimizedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 10/22/2011 20:11'!
serializeInstances: aSerialization! !

!FLOptimizedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 10/22/2011 20:22'!
serializeReferencesOf: anObject with: aSerialization
	^ self subclassResponsibility ! !

!FLOptimizedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 10/22/2011 20:18'!
serializeReferencesStepWith: aSerialization

	objects do: [ :anObject | self serializeReferencesOf: anObject with: aSerialization ]! !

FLOptimizedObjectCluster subclass: #FLPointCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLPointCluster commentStamp: 'MarianoMartinezPeck 9/8/2011 22:33' prior: 0!
FLPointCluster is an optional class that optimizes Point instances, since there are a lot of instances in the system, it makes sense to optimize them. We take advantage of bytecode optimizations done by Pharo for messages like #@, #x and #y. In addition, we avoid the overhead of class reshape, etc. !

!FLPointCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization
	"Since in Pharo #@ is associated with a special bytecode, it is faster than doing  Point basicNew"
	^ 0@0 
	! !

!FLPointCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 10/22/2011 20:23'!
materializeReferencesOf: aPoint with: aMaterialization

		aPoint setX: aMaterialization nextEncodedReference setY: aMaterialization nextEncodedReference.	
! !

!FLPointCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/8/2011 21:37'!
referencesOf: aPoint do: aBlock

	aBlock value: aPoint x.
	aBlock value: aPoint y.
	! !

!FLPointCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 10/22/2011 20:21'!
serializeReferencesOf: aPoint with: aSerialization

		aSerialization encodeReferenceTo: aPoint x.
		aSerialization encodeReferenceTo: aPoint y.
! !

FLOptimizedObjectCluster subclass: #FLRectangleCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLRectangleCluster commentStamp: 'MarianoMartinezPeck 9/8/2011 22:53' prior: 0!
FLRectangleCluster is an optional class that optimizes Rectangle instances, since there are a lot of instances in the system, it makes sense to optimize them. We take advantage of bytecode optimizations done by Pharo for messages like #new, #@, #x and #y. In addition, we avoid the overhead of class reshape, etc. !

!FLRectangleCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization
	"Since in Pharo #@ is associated with a special bytecode, it is faster than doing  Point basicNew"
	^ Rectangle basicNew! !

!FLRectangleCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 10/22/2011 20:23'!
materializeReferencesOf: aRectangle with: aMaterialization
		
		aRectangle 
			setOrigin: aMaterialization nextEncodedReference @ aMaterialization nextEncodedReference	
		 	corner: aMaterialization nextEncodedReference @ aMaterialization nextEncodedReference.	
	 ! !

!FLRectangleCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/8/2011 22:45'!
referencesOf: aRectangle do: aBlock

	aBlock value: aRectangle origin x.
	aBlock value: aRectangle origin y.
	aBlock value: aRectangle corner x.
	aBlock value: aRectangle corner y.! !

!FLRectangleCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 10/22/2011 20:21'!
serializeReferencesOf: aRectangle with: aSerialization

		aSerialization encodeReferenceTo: aRectangle origin x.
		aSerialization encodeReferenceTo: aRectangle origin y.
		aSerialization encodeReferenceTo: aRectangle corner x.
		aSerialization encodeReferenceTo: aRectangle corner y.
! !

FLIteratingCluster subclass: #FLPrimitiveCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPrimitiveCluster commentStamp: 'MartinDias 8/29/2011 19:21' prior: 0!
I am a cluster that serializes and materializes interacting directly with the stream.!

FLPrimitiveCluster subclass: #FLGlobalCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalCluster commentStamp: 'MartinDias 9/16/2011 14:49' prior: 0!
I am a cluster for objects that are reachables from Smalltalk global dictionary.!

FLGlobalCluster subclass: #FLGlobalAssociationCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalAssociationCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for variable bindings that belong to Smalltalk global dictionary.

For example, 

	Smalltalk globals associationAt: #Transcript 
	
could be serialized and materialized by me.!

!FLGlobalAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeFrom: aReadStream

	^self materializeGlobalAssociationFrom: aReadStream! !

!FLGlobalAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serialize: anAssociation on: aWriteStream
	
	self serializeGlobalAssociationKeyed: anAssociation key on: aWriteStream ! !

FLGlobalCluster subclass: #FLGlobalClassCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalClassCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for classes that should be present when materializing. In other words, it only serializes a reference to the class, using its name.!

!FLGlobalClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeFrom: aReadStream

	^self materializeGlobalClassFrom: aReadStream ! !

!FLGlobalClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serialize: aClass on: aWriteStream
	
	self serializeGlobalClass: aClass on: aWriteStream! !

FLGlobalCluster subclass: #FLGlobalClassSideCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalClassSideCluster commentStamp: 'MartinDias 8/1/2011 02:57' prior: 0!
I am a cluster for those Metaclasses and ClassTrait who are not stored in detail.!

!FLGlobalClassSideCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeFrom: aReadStream

	^(self materializeGlobalClassFrom: aReadStream) classSide! !

!FLGlobalClassSideCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serialize: aClassOrTraitClassSide on: aWriteStream

	self serializeGlobalClass: aClassOrTraitClassSide instanceSide on: aWriteStream
! !

!FLGlobalCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/15/2011 20:16'!
clusterBucketIn: aClusterization

	^aClusterization globalsBucket ! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/24/2011 17:55'!
materializeGlobalAssociationFrom: aStream

	| globalName |
	globalName := aStream nextString asSymbol.
	^Smalltalk globals 
		associationAt: globalName
		ifAbsent: [ FLGlobalNotFound signalWithName: globalName ]
! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/24/2011 17:52'!
materializeGlobalClassFrom: aStream

	| className |
	className := aStream nextString asSymbol.
	^ Smalltalk globals 
		at: className
		ifAbsent: [ FLClassNotFound signalWithName: className ]! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serializeGlobalAssociationKeyed: aSymbol on: aStream
	
	aStream nextStringPut: aSymbol! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/16/2011 01:37'!
serializeGlobalClass: aClass on: aStream
	
	aClass isObsolete ifTrue: [ FLObsolete signalWithName: aClass name ].
	aStream nextStringPut: aClass name ! !

FLGlobalCluster subclass: #FLGlobalCompiledMethodCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/29/2011 20:59'!
materializeFrom: aReadStream

	| methodClass selector |
	methodClass := self materializeGlobalClassFrom: aReadStream.
	(aReadStream next = 1) ifTrue: [
		methodClass := methodClass classSide].
	selector := aReadStream nextString asSymbol.
	^methodClass  
		compiledMethodAt: selector
		ifAbsent: [FLMethodNotFound signalWith: methodClass name and: selector]! !

!FLGlobalCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/28/2011 10:53'!
serialize: aCompiledMethod on: aWriteStream
	| methodClass |
	methodClass := aCompiledMethod methodClass.
	self serializeGlobalClass: methodClass instanceSide on: aWriteStream.
	aWriteStream nextPut: (methodClass isClassSide ifTrue: 1 ifFalse: 0).
	aWriteStream nextStringPut: aCompiledMethod selector.! !

FLGlobalCluster subclass: #FLGlobalSendCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalSendCluster commentStamp: 'MarianoMartinezPeck 10/23/2011 14:39' prior: 0!
I clusterize objects that will be obtained on materialization via a message send to a global object. 

Explained with an example:

Suppose we have a special instance of User that represents the admin user, and it is a unique instance in the image. In case the admin user is referenced in our graph, we want to treat that object as a global. We can do that in this way:

User >> fuelAccept: aVisitor
    ^self == User admin
        ifTrue: [aVisitor visitGlobalSend: self]
        ifFalse: [super fuelAccept: aVisitor]

User >> fuelGlobalName
    ^#User

User >> fuelSelector
    ^#admin

So what will happen is that during serialization, the admin user won't be completly serialized (with all its intance variables) but instead its global name and selector are stored. Then, at materialization time, Fuel will send the selector #admin to the class User, and use what that answers as the admin user of the materialized graph.

We test this feature in FLGlobalSendSerializationTest.!

!FLGlobalSendCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/5/2011 15:36'!
materializeFrom: aReadStream

	| global selector |
	global := (self materializeGlobalAssociationFrom: aReadStream) value.	
	selector := aReadStream nextString asSymbol.
	^ global perform: selector! !

!FLGlobalSendCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/5/2011 15:43'!
serialize: anObject on: aWriteStream
	
	self serializeGlobalAssociationKeyed: anObject fuelGlobalName on: aWriteStream.
	aWriteStream nextStringPut: anObject fuelSelector.
! !

FLGlobalCluster subclass: #FLGlobalValueCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalValueCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for known objects that belong to Smalltalk global dictionary.

For example, Transcript could be serialized and materialized by me.!

!FLGlobalValueCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeFrom: aReadStream

	^(self materializeGlobalAssociationFrom: aReadStream) value! !

!FLGlobalValueCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serialize: anObject on: aWriteStream
	
	self serializeGlobalAssociationKeyed: (Smalltalk keyAtValue: anObject) on: aWriteStream! !

!FLPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 10/13/2011 16:26'!
add: anObject traceWith: aAnalysis
	"Add an object to the cluster. We know the object doesn't have references."

	objects addLast: anObject! !

!FLPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:28'!
clusterBucketIn: aClusterization

	^aClusterization primitivesBucket ! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:30'!
materializeFrom: aReadStream
	"Hook method that each subclass should customize for its special way of materializing"
	
	self subclassResponsibility ! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization
	
	^self materializeFrom: aMaterialization stream.  ! !

!FLPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/19/2011 14:06'!
newSerializingCollection
	"Answer a collection for the objects that correspond to this cluster."

	^OrderedCollection new! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:26'!
registerIndexesOn: aDictionary

	self flag: #todo. "Converting objects collection here in this method is a bit confusing. This is because since this cluster is for primitives, they do not have pointers to other objects. Hence, instead of storing them in a IdentitySet we can use an OrderedCollection and then just convert them at the end. For more details see FLPrimitiveCluster >> #add: anObject traceWith: aAnalysis"
	objects := objects asIdentitySet.
	super registerIndexesOn: aDictionary.! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:29'!
serialize: anObject on: aWriteStream
	"Hook method that each subclass should customize for its special way of serializing"
	
	self subclassResponsibility! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: anObject with: aSerialization

	self serialize: anObject on: aSerialization stream! !

FLPrimitiveCluster subclass: #FLSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSmallIntegerCluster commentStamp: '<historical>' prior: 0!
I don't have so much sense as a class, but I know which of my subclasses corresponds to each SmallInteger.!

FLSmallIntegerCluster subclass: #FLNegativeSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLNegativeSmallIntegerCluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for negative SmallInteger instances.!

!FLNegativeSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'mariano 10/6/2011 11:43'!
materializeFrom: aReadStream
	
	^ aReadStream uint32 negated ! !

!FLNegativeSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'mariano 10/6/2011 11:43'!
serialize: anInteger on: aWriteStream
	
	aWriteStream uint32: anInteger abs! !

FLSmallIntegerCluster subclass: #FLPositiveSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPositiveSmallIntegerCluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for positive SmallInteger instances.!

!FLPositiveSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'mariano 10/6/2011 11:43'!
materializeFrom: aReadStream

	^ aReadStream uint32! !

!FLPositiveSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'mariano 10/6/2011 11:43'!
serialize: anInteger on: aWriteStream

	aWriteStream uint32: anInteger! !

!FLSmallIntegerCluster class methodsFor: 'instance creation' stamp: 'mariano 10/6/2011 17:30'!
for: aSmallInteger
	"Returns the right subclass instance"

	^ ((aSmallInteger bitAnd: 16rFFFF) = aSmallInteger)
		ifTrue: [ ^ FLUInt16Cluster new ]
		ifFalse: [ ^ FLPositiveSmallIntegerCluster new]
	! !

FLSmallIntegerCluster subclass: #FLUInt16Cluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLUInt16Cluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for SmallInteger instances between 0 and 16rFFFF.!

!FLUInt16Cluster methodsFor: 'serialize/materialize' stamp: 'mariano 10/6/2011 11:44'!
materializeFrom: aReadStream

	^ aReadStream uint16! !

!FLUInt16Cluster methodsFor: 'serialize/materialize' stamp: 'mariano 10/6/2011 11:44'!
serialize: anInteger on: aWriteStream

	aWriteStream uint16: anInteger! !

FLCluster subclass: #FLSubstitutionCluster
	instanceVariableNames: 'substitutions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSubstitutionCluster commentStamp: 'MartinDias 8/29/2011 19:25' prior: 0!
I am a cluster for objects that have to be replaced in the object graph by a substitute object.!

FLSubstitutionCluster subclass: #FLHookSubstitutionCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLHookSubstitutionCluster commentStamp: 'MartinDias 8/29/2011 19:30' prior: 0!
I am a cluster for substituted objects, where such substitution is provided by a method hook.

Suppose you want to substitute instances of WriteStream by nil. In such case, WriteStream has to implement these two methods:

fuelAccept: aVisitor
	^aVisitor visitSubstitution: self

fuelSubstitution
	^nil!

!FLHookSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:46'!
substitutionFor: anObject
	
	^anObject fuelSubstitution! !

FLSubstitutionCluster subclass: #FLPluggableSubstitutionCluster
	instanceVariableNames: 'factory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPluggableSubstitutionCluster commentStamp: 'MartinDias 8/29/2011 19:32' prior: 0!
I am a cluster for substituted objects, where such substitution is provided by a factory block.

Suppose you want to substitute every integer in the graph by its string representation. In such case you should configure the analyzer through:

anAnalyzer when: [:x | x isInteger] substituteBy: [:x | x printString].

In this way, when you serialize #(1 2), you will then materialize #('1' '2')!

!FLPluggableSubstitutionCluster classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 10/24/2011 16:41'!
with: aFactory

	^self basicNew
		initializeWith: aFactory;
		yourself.! !

!FLPluggableSubstitutionCluster methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/24/2011 16:42'!
initializeWith: aFactory
	super initialize.
	factory := aFactory! !

!FLPluggableSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:45'!
substitutionFor: anObject
	
	^factory value: anObject! !

!FLSubstitutionCluster methodsFor: 'analyzing' stamp: 'MartinDias 10/13/2011 16:26'!
add: anObject traceWith: aAnalysis

	substitutions 
		at: anObject 
		ifAbsent: [ 
			| substitution |
			substitution := self substitutionFor: anObject.
			substitutions at: anObject put: substitution.
			aAnalysis trace: substitution ] ! !

!FLSubstitutionCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:50'!
clusterBucketIn: aClusterization

	^aClusterization substitutionsBucket! !

!FLSubstitutionCluster methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 23:17'!
initialize

	super initialize.
	substitutions := IdentityDictionary new.! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:22'!
materializeInstancesStepWith: aMaterialization
	"This cluster does nothing."! !

!FLSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/29/2011 01:04'!
objects
	"This cluster does not have objects"
	
	^#()! !

!FLSubstitutionCluster methodsFor: 'printing' stamp: 'MartinDias 8/29/2011 01:16'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '->'.
	substitutions printElementsOn: aStream! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/20/2011 23:18'!
registerIndexesOn: aDictionary

	substitutions keysAndValuesDo: [ :anObject :theSubstitution | 
		| substitutionIndex |
		substitutionIndex := aDictionary at: theSubstitution.
		aDictionary at: anObject put: substitutionIndex. ].! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstancesStepWith: aSerialization
	"This cluster does nothing."! !

!FLSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:45'!
substitutionFor: anObject
	
	self subclassResponsibility! !

FLCluster subclass: #FLWellKnownObjectsCluster
	instanceVariableNames: ''
	classVariableNames: 'WellKnownObjects'
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLWellKnownObjectsCluster commentStamp: 'MartinDias 8/1/2011 03:00' prior: 0!
I am a cluster for some well known objects like nil, true and false. They have a special treatment because they are so common in the image.!

!FLWellKnownObjectsCluster classSide methodsFor: 'class initialization' stamp: 'MartinDias 8/30/2011 00:53'!
initialize

	WellKnownObjects := Array
		with: nil
		with: true
		with: false
		with: Smalltalk globals! !

!FLWellKnownObjectsCluster methodsFor: 'analyzing' stamp: 'MartinDias 10/13/2011 16:26'!
add: anObject traceWith: aAnalysis

	"Do nothing. THis is an optimization. As you see, anObject is not added. In the conrary, #registerIndexesOn: directly serializes all the array of WellKnownObjects. See #objects"! !

!FLWellKnownObjectsCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:28'!
clusterBucketIn: aClusterization

	^aClusterization primitivesBucket! !

!FLWellKnownObjectsCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:22'!
materializeInstancesStepWith: aMaterialization
! !

!FLWellKnownObjectsCluster methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/28/2011 23:15'!
objects
	" This is an optimization. We always serialize all the WellKnownObjects"
	^WellKnownObjects! !

!FLWellKnownObjectsCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/30/2011 00:46'!
registerIndexesOn: aDictionary

	self objects do: [ :instance | aDictionary at: instance put: aDictionary size + 1 ].! !

!FLWellKnownObjectsCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstancesStepWith: aSerialization
! !

Object subclass: #FLClusterization
	instanceVariableNames: 'baselevelBucket substitutionsBucket globalsBucket metalevelInstanceSideBucket metalevelClassSideBucket primitivesBucket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLClusterization commentStamp: 'MartinDias 8/29/2011 19:12' prior: 0!
I manage the clusters collected during the analysis step of serialization.!

!FLClusterization methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:16'!
add: aCluster traceWith: anAnalyzer

	| clusterBucket |
	clusterBucket := aCluster clusterBucketIn: self.
	
	clusterBucket 
		addIfNotPresent: aCluster 
		ifPresentDo: [ ^self ].

	aCluster clusterReferencesDo: [ :aChild | anAnalyzer trace: aChild ]! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:27'!
baselevelBucket
	
	^ baselevelBucket! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 9/17/2011 00:29'!
clusterBuckets
	"Answer a new collection with all the clusters that have been collected in trace. Note that order is imporant."

	^ OrderedCollection new 
		add: globalsBucket;
		add: primitivesBucket;
		add: metalevelClassSideBucket;
		add: metalevelInstanceSideBucket;
		add: baselevelBucket;
		add: substitutionsBucket;
		yourself.! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:47'!
clusters
	"Returns collection with all the clustered objects, in Association instances."
	
	^self clusterBuckets inject: OrderedCollection new into: [:c :d | c addAll: d. c ].
! !

!FLClusterization methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:44'!
fuelAccept: aVisitor  
	^ aVisitor visitSubstitution: self! !

!FLClusterization methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:44'!
fuelSubstitution 
	^ nil! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 9/15/2011 20:16'!
globalsBucket
	
	^ globalsBucket ! !

!FLClusterization methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 9/23/2011 23:15'!
initialize

	super initialize.
	primitivesBucket := IdentitySet new.
	baselevelBucket := IdentitySet new.
	globalsBucket := IdentitySet new.
	metalevelInstanceSideBucket := IdentitySet new.
	metalevelClassSideBucket := IdentitySet new.
	substitutionsBucket := IdentitySet new.
! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 9/17/2011 00:27'!
metalevelClassSideBucket
	
	^ metalevelClassSideBucket! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 9/17/2011 00:27'!
metalevelInstanceSideBucket
	
	^ metalevelInstanceSideBucket ! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 22:49'!
objectCount

	^self clusterBuckets sum: [ :aClusterBucket | 
		aClusterBucket
			inject: 0 
			into: [ :sum :aCluster | sum + aCluster objects size ] ].! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 9/17/2011 00:29'!
primitivesBucket
	
	^ primitivesBucket! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:52'!
substitutionsBucket
	
	^ substitutionsBucket! !

Object subclass: #FLCompiledMethodTrailerEncoderNoChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCompiledMethodTrailerEncoderNoChange methodsFor: 'method encoding' stamp: 'MarianoMartinezPeck 9/23/2011 16:37'!
compiledMethodToSerialize: aCompiledMethod
	^ aCompiledMethod! !

Object subclass: #FLCompiledMethodTrailerEncoderNoSource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCompiledMethodTrailerEncoderNoSource methodsFor: 'method encoding' stamp: 'MarianoMartinezPeck 9/23/2011 23:21'!
compiledMethodToSerialize: aCompiledMethod

	^ aCompiledMethod copyWithTrailerBytes: CompiledMethodTrailer empty.! !

Object subclass: #FLMapper
	instanceVariableNames: 'next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLMapper commentStamp: 'MartinDias 8/11/2011 03:09' prior: 0!
The purpose of my hierarchy is to map objects with clusters.!

FLMapper subclass: #FLLightGeneralMapper
	instanceVariableNames: 'wellKnownObjectCluster characterCluster floatCluster shortCluster symbolCluster byteObjectClusters variableObjectClusters fixedObjectClusters wordObjectClusters byteStringCluster hookSubstitutionCluster dateCluster timeCluster durationCluster dateAndTimeCluster bitmapCluster pointCluster rectangleCluster globalSendCluster positiveSmallIntegerCluster negativeSmallIntegerCluster globalClassSideCluster globalInstanceSideCluster globalCompiledMethodCluster blockClosureCluster weakVariableObjectClusters compiledMethodCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLLightGeneralMapper methodsFor: 'accessing' stamp: 'MartinDias 11/27/2011 12:51'!
accepts: anObject

	^ true! !

!FLLightGeneralMapper methodsFor: 'accessing' stamp: 'MartinDias 11/27/2011 12:51'!
chainedClusterFor: anObject

	^anObject fuelAccept: self! !

!FLLightGeneralMapper methodsFor: 'accessing' stamp: 'MartinDias 11/27/2011 12:51'!
clusterFor: anObject

	^anObject fuelAccept: self
! !

!FLLightGeneralMapper methodsFor: 'initialize-release' stamp: 'MartinDias 12/14/2011 16:05'!
initialize

	super initialize.
	fixedObjectClusters := IdentityDictionary new.
	variableObjectClusters := IdentityDictionary new.
	byteObjectClusters := IdentityDictionary new.
	wordObjectClusters := IdentityDictionary new.
	weakVariableObjectClusters := IdentityDictionary new.

	globalCompiledMethodCluster := FLGlobalCompiledMethodCluster new.
	globalClassSideCluster := FLGlobalClassSideCluster new.
	globalInstanceSideCluster := FLGlobalClassCluster new.
	globalSendCluster := FLGlobalSendCluster new.	
	wellKnownObjectCluster := FLWellKnownObjectsCluster new.
	characterCluster := FLHookPrimitiveCluster for: Character.
	floatCluster := FLHookPrimitiveCluster for: Float.
	shortCluster := FLUInt16Cluster new.
	positiveSmallIntegerCluster := FLPositiveSmallIntegerCluster new.
	negativeSmallIntegerCluster := FLNegativeSmallIntegerCluster new.
	symbolCluster := FLHookPrimitiveCluster for: ByteSymbol.
	byteStringCluster := FLHookPrimitiveCluster for: ByteString.
	hookSubstitutionCluster := FLHookSubstitutionCluster new.
	dateCluster := FLHookPrimitiveCluster for: Date.
	timeCluster := FLHookPrimitiveCluster for: Time. 
	durationCluster := FLHookPrimitiveCluster for: Duration.
	dateAndTimeCluster := FLHookPrimitiveCluster for: DateAndTime.
	bitmapCluster := FLHookPrimitiveCluster for: Bitmap.
	pointCluster := FLPointCluster new.
	rectangleCluster := FLRectangleCluster new.
	blockClosureCluster := FLBlockClosureCluster new.
	compiledMethodCluster := FLCompiledMethodCluster withNoTrailerChange.! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitBitmap: aBitmap

	^bitmapCluster
	! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 12/3/2011 00:15'!
visitBlockClosure: aBlockClosure 
	^ blockClosureCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitBoolean: aBoolean

	^wellKnownObjectCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitByteString: aSymbol

	^byteStringCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitBytesObject: anObject

	^byteObjectClusters
		at: anObject class
		ifAbsentPut: [ FLByteObjectCluster for: anObject class ]! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitCharacter: aCharacter

	^aCharacter isOctetCharacter 
		ifTrue: [ characterCluster ]
		ifFalse: [ 
			fixedObjectClusters
				at: aCharacter class
				ifAbsentPut: [ FLFixedObjectCluster for: aCharacter class ]]! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:55'!
visitClass: aClass

	^globalInstanceSideCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:55'!
visitClassTrait: aClassTrait

	^globalClassSideCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 12/11/2011 00:06'!
visitCompiledMethod: aCompiledMethod

	^ aCompiledMethod selector isDoIt	
		ifTrue:  [  compiledMethodCluster ]
		ifFalse: [ globalCompiledMethodCluster ]
! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitDate: aDate

	^dateCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitDateAndTime: aDateAndTime

	^dateAndTimeCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitDuration: aTime

	^durationCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitFixedObject: anObject

	^fixedObjectClusters
		at: anObject class
		ifAbsentPut: [ FLFixedObjectCluster for: anObject class ]! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitFloat: aFloat

	^floatCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitGlobalSend: anObject

	^globalSendCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:55'!
visitMetaclass: aMetaclass

	^globalClassSideCluster ! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitMethodContext: aMethodContext

	^variableObjectClusters
		at: aMethodContext class
		ifAbsentPut: [ FLMethodContextCluster for: aMethodContext class ]
! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitPoint: aDate

	^pointCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitRectangle: aRectangle

	^ rectangleCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitSmallInteger: aSmallInteger

    
	((aSmallInteger bitAnd: 16rFFFF) = aSmallInteger)
		ifTrue: [ ^shortCluster ].

	^aSmallInteger > 0 
		ifTrue: [ positiveSmallIntegerCluster ]
		ifFalse: [ negativeSmallIntegerCluster ]
		! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitSubstitution: anObject

	^hookSubstitutionCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitSymbol: aSymbol

	^symbolCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitSystemDictionary: anObject

	^wellKnownObjectCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitTime: aTime

	^timeCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:56'!
visitTrait: aTrait

	^globalInstanceSideCluster ! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitUndefinedObject: anObject

	^wellKnownObjectCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitVariableObject: anObject

	^variableObjectClusters
		at: anObject class
		ifAbsentPut: [ FLVariableObjectCluster for: anObject class ]! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 12/5/2011 22:33'!
visitWeakObject: anObject

	^weakVariableObjectClusters
		at: anObject class
		ifAbsentPut: [ FLWeakVariableObjectCluster for: anObject class ]! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 12/13/2011 02:35'!
visitWellKnownObject: anObject

	^wellKnownObjectCluster! !

!FLLightGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 11/27/2011 12:51'!
visitWordsObject: anObject

	^wordObjectClusters
		at: anObject class
		ifAbsentPut: [ FLWordObjectCluster for: anObject class ]! !

FLMapper subclass: #FLLightGlobalMapper
	instanceVariableNames: 'globalCluster'
	classVariableNames: 'Globals'
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLLightGlobalMapper commentStamp: 'MartinDias 8/20/2011 23:43' prior: 0!
I map classes, traits, and global objects that belong to Smalltalk dictionary. I am included in FLAnalyzer newLight behavior. 

I am a fast simplified replace to this four clusters included in FLAnaluzer newFull behavior: 
- FLGlobalMapper
- FLGlobalBehaviorMapper
- FLClassPoolAssociationMapper
- FLGlobalAssociationMapper 

For example, the global Transcript as well as any class in the image, are mapped by me.!

!FLLightGlobalMapper class methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:12'!
initialize

	self initializeGlobals! !

!FLLightGlobalMapper class methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:20'!
initializeGlobals

	Globals := IdentitySet new
		add: Display;
		add: World;
		add: Smalltalk;
		add: Transcript;
		add: SourceFiles;
		add: Processor;
		add: ImageImports;
		add: TextConstants;
		add: Undeclared;
		add: SystemOrganization;
		add: Sensor;
		add: ActiveWorld;
		add: ActiveHand;
		yourself! !

!FLLightGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 11/27/2011 13:07'!
chainedClusterFor: anObject
	"Uses the Chain of Responsibility pattern to answer the cluster which maps with the received object."

	(Globals includes: anObject) ifTrue: [ ^globalCluster ].
	^next chainedClusterFor: anObject.! !

!FLLightGlobalMapper methodsFor: 'initialize-release' stamp: 'MartinDias 11/27/2011 13:07'!
initialize

	super initialize.
	globalCluster := FLGlobalValueCluster new.! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 02:01'!
accepts: anObject
	"Tells if the received object is handled by this analyzer"

	self subclassResponsibility ! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 20:21'!
chainedClusterFor: anObject
	"Uses the Chain of Responsibility pattern to answer the cluster which maps with the received object."

	^(self accepts: anObject)
		ifTrue: [ self clusterFor: anObject ]
		ifFalse: [ next chainedClusterFor: anObject ].
! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 20:19'!
clusterFor: anObject
	"Answers the cluster which maps with the received object."

	self subclassResponsibility ! !

!FLMapper methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:47'!
fuelAccept: aVisitor  
	^ aVisitor visitSubstitution: self! !

!FLMapper methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:47'!
fuelSubstitution 
	^ nil! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 20:37'!
internalObjects: aCollection! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 17:19'!
next
	^ next! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 17:19'!
next: anObject
	next := anObject! !

FLMapper subclass: #FLPluggableSubstitutionMapper
	instanceVariableNames: 'condition substitutionCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLPluggableSubstitutionMapper commentStamp: 'MartinDias 8/20/2011 23:45' prior: 0!
I map substituted objects. 

For example, suppose you want to substitute instances of WriteStream by nil. In such case, I will map every WriteStream instance to my substitutions cluster, and its factory block will value to nil with any argument.
!

!FLPluggableSubstitutionMapper class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 10/24/2011 16:43'!
when: aCondition substituteBy: aFactory

	^self basicNew
		initializeWith: aCondition substitutionCluster: (FLPluggableSubstitutionCluster with: aFactory);
		yourself! !

!FLPluggableSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:37'!
accepts: anObject

	^condition value: anObject! !

!FLPluggableSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 20:31'!
clusterFor: anObject

	^substitutionCluster! !

!FLPluggableSubstitutionMapper methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/24/2011 16:44'!
initializeWith: aCondition substitutionCluster: aSubstitutionCluster
	super initialize.
	condition := aCondition.
	substitutionCluster := aSubstitutionCluster ! !

Object subclass: #FLMaterialization
	instanceVariableNames: 'stream clusterCount indexCluster clusters root objects isBigEndian'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLMaterialization commentStamp: 'MarianoMartinezPeck 10/23/2011 14:42' prior: 0!
I implement the algorithm for materializing an object graph on a stream. FLMaterializer known how to build instances of me.!

!FLMaterialization class methodsFor: 'instance creation' stamp: 'MartinDias 9/10/2011 18:32'!
on: aReadStream

	^self basicNew
		initializeWith: aReadStream;
		yourself
! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
afterMaterializationStep

	clusters do: [ :aCluster | aCluster afterMaterializationStepWith: self ]! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
clusterInstancesStep

	| aCluster |
	aCluster := self nextEncodedCluster.
	aCluster materializeInstancesStepWith: self.
	self registerAll: aCluster objects.
	aCluster materializePostInstancesStepWith: self.
	clusters add: aCluster.! !

!FLMaterialization methodsFor: 'private' stamp: 'mariano 10/6/2011 17:10'!
headerStep

	indexCluster := self nextEncodedCluster.
	objects := OrderedCollection new: self nextEncodedPositiveInteger.
	objects resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast. In fact, this has already been fixed in Pharo 1.4: http://code.google.com/p/pharo/issues/detail?id=1686"

	clusterCount := self nextEncodedPositiveInteger.
	clusters := OrderedCollection new: clusterCount.
	clusters resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast"
	
	isBigEndian := self nextEncodedPositiveInteger.
! !

!FLMaterialization methodsFor: 'initialize-release' stamp: 'MartinDias 9/10/2011 18:33'!
initializeWith: aReadStream

	self initialize.
	stream := aReadStream.! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
instancesStep

	clusterCount timesRepeat: [ self clusterInstancesStep ]
! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
isBigEndian
	^ isBigEndian ! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/27/2011 17:17'!
materializedInstances
	^ objects! !

!FLMaterialization methodsFor: 'decoding' stamp: 'MartinDias 9/9/2011 01:58'!
nextEncodedCluster

	^(Smalltalk classNamed: stream nextString) new! !

!FLMaterialization methodsFor: 'decoding' stamp: 'MartinDias 9/11/2011 19:33'!
nextEncodedPositiveInteger
	
	^ stream uint32! !

!FLMaterialization methodsFor: 'decoding' stamp: 'mariano 10/6/2011 17:02'!
nextEncodedReference 

	^ objects at: (indexCluster materializeFrom: stream)! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
objects
	"Answer a collection with the materialized objects."

	^ objects ! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
referencesStep

	clusters do: [ :aCluster | aCluster materializeReferencesStepWith: self ]! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
registerAll: materializedObjects
	
	objects addAll: materializedObjects ! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
root

	^ root ! !

!FLMaterialization methodsFor: 'materializing' stamp: 'MarianoMartinezPeck 9/27/2011 16:50'!
run

	self headerStep.
	self instancesStep.
	self referencesStep.
	self trailerStep.
	self afterMaterializationStep.! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
stream
	^ stream! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
trailerStep

	root := self nextEncodedReference! !

Object subclass: #FLMaterializer
	instanceVariableNames: 'materializationFactory streamFactory streamFinalizer signature version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLMaterializer commentStamp: 'MartinDias 8/29/2011 19:06' prior: 0!
I am a binary object materializer. 

See an example of use in FLSerializer's documentation.

!

!FLMaterializer class methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/9/2011 14:26'!
currentVersion
	"If you change this method, you should also create a version in ConfigurationOfFuel and FLSerializer >> currentVersion"
	^ 17! !

!FLMaterializer class methodsFor: 'protected' stamp: 'MartinDias 10/6/2011 23:31'!
defaultSignature
	^ 'FUEL'! !

!FLMaterializer class methodsFor: 'materializing-shortcuts' stamp: 'MarianoMartinezPeck 9/27/2011 17:14'!
materializationFromFileNamed: aFilename

	^StandardFileStream 
		oldFileNamed: aFilename 
		do: [:aFileStream | 
			(self newDefault materializeFrom: aFileStream binary) ]
! !

!FLMaterializer class methodsFor: 'materializing-shortcuts' stamp: 'MartinDias 9/9/2011 20:25'!
materializeFromByteArray: byteArray

	^(self newDefault materializeFrom: byteArray readStream) root! !

!FLMaterializer class methodsFor: 'materializing-shortcuts' stamp: 'MarianoMartinezPeck 9/27/2011 17:15'!
materializeFromFileNamed: aFilename

	^ (self materializationFromFileNamed: aFilename) root
! !

!FLMaterializer class methodsFor: 'instance creation' stamp: 'MartinDias 9/10/2011 18:16'!
newDefault

	^self new! !

!FLMaterializer methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:49'!
fuelAccept: aVisitor  
	^ aVisitor visitSubstitution: self! !

!FLMaterializer methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:49'!
fuelSubstitution 
	^ nil! !

!FLMaterializer methodsFor: 'initialize-release' stamp: 'MartinDias 10/6/2011 23:32'!
initialize

	super initialize.
	self setDefaultMaterialization.
	self readDirectly.
	self signature: self class defaultSignature.
	self version: self class currentVersion.! !

!FLMaterializer methodsFor: 'materializing' stamp: 'MarianoMartinezPeck 10/22/2011 12:51'!
materializeFrom: sourceStream

	| aStream |
	aStream := streamFactory value: sourceStream.
	[	self verifySignatureFrom: aStream.
		self verifyVersionFrom: aStream.
		^ materializationFactory value: aStream ] ensure: [ 
		streamFinalizer value: aStream ]! !

!FLMaterializer methodsFor: 'configuring-stream' stamp: 'MartinDias 9/27/2011 20:31'!
readDirectly

	streamFactory := [:aStream | aStream ].
	streamFinalizer := [:aStream | ].! !

!FLMaterializer methodsFor: 'protected' stamp: 'MartinDias 9/14/2011 00:23'!
setDefaultMaterialization

	materializationFactory := [:aStream | (FLMaterialization on: aStream) run; yourself ]
	
	
! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 22:57'!
signature
	^ signature! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 22:57'!
signature: anObject
	signature := anObject! !

!FLMaterializer methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/23/2011 14:19'!
verifySignatureFrom: aStream

	| streamSignature |
	streamSignature := aStream next: self signature size.
	(self signature asByteArray = streamSignature) ifFalse: [ FLBadSignature signalCurrentSignature: self signature streamSignature: streamSignature  ].
! !

!FLMaterializer methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/23/2011 14:03'!
verifyVersionFrom: aStream

	| streamVersion |
	streamVersion := aStream uint16.
	(self version = streamVersion) ifFalse: [ FLBadVersion signalCurrentVersion: self version streamVersion: streamVersion ]
! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 22:57'!
version
	^ version! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 22:57'!
version: anObject
	version := anObject! !

Object subclass: #FLSerialization
	instanceVariableNames: 'stream indexCluster root objectsIndexes clusters objectCount analyzer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLSerialization commentStamp: 'MarianoMartinezPeck 10/23/2011 14:42' prior: 0!
I implement the algorithm for serializing an object graph on a stream. FLSerializer known how to build instances of me.!

!FLSerialization class methodsFor: 'instance creation' stamp: 'MartinDias 9/10/2011 18:06'!
root: anObject on: aWriteStream analyzer: anAnalyzer
	
	^ self basicNew
		initializeRoot: anObject
		on: aWriteStream
		analyzer: anAnalyzer;
		yourself! !

!FLSerialization methodsFor: 'private' stamp: 'MarianoMartinezPeck 12/18/2011 22:54'!
analysisStep

	| anAnalysis |
	anAnalysis := analyzer analysisFor: root.
	clusters := anAnalysis clusterization clusters.
	objectCount := anAnalysis clusterization objectCount.
	objectsIndexes := LargeIdentityDictionary new.
	indexCluster := FLSmallIntegerCluster for: objectCount.! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:11'!
clusterInstancesStepOf: aCluster

	self encodeIdOf: aCluster.
	aCluster serializeInstancesStepWith: self.
	aCluster serializePostInstancesStepWith: self.
! !

!FLSerialization methodsFor: 'accessing' stamp: 'MartinDias 9/13/2011 16:34'!
clusters
	^ clusters! !

!FLSerialization methodsFor: 'encoding' stamp: 'MartinDias 9/9/2011 01:11'!
encodeIdOf: aCluster

	stream nextStringPut: aCluster class name! !

!FLSerialization methodsFor: 'encoding' stamp: 'MartinDias 9/11/2011 19:33'!
encodePositiveInteger: anInteger

	stream uint32: anInteger! !

!FLSerialization methodsFor: 'encoding' stamp: 'MarianoMartinezPeck 12/6/2011 14:43'!
encodeReferenceTo: anObject

	indexCluster 
		serialize: (objectsIndexes at: anObject) 
		on: stream! !

!FLSerialization methodsFor: 'encoding' stamp: 'MarianoMartinezPeck 12/6/2011 00:19'!
encodeWeakReferenceTo: anObject

	indexCluster 
		serialize: (objectsIndexes at: anObject ifAbsent: [objectsIndexes at: nil]) 
		on: stream! !

!FLSerialization methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:40'!
fuelAccept: aVisitor  
	^ aVisitor visitSubstitution: self! !

!FLSerialization methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:40'!
fuelSubstitution 
	^ nil! !

!FLSerialization methodsFor: 'private' stamp: 'MarianoMartinezPeck 10/6/2011 21:14'!
headerStep
	
	self encodeIdOf: indexCluster.
	self encodePositiveInteger: objectCount.
	self encodePositiveInteger: clusters size.
	self encodePositiveInteger: Smalltalk isBigEndian asBit.! !

!FLSerialization methodsFor: 'initialize-release' stamp: 'MartinDias 9/12/2011 00:12'!
initializeRoot: anObject on: aWriteStream analyzer: anAnalyzer
	
	self initialize.
	root := anObject.
	stream := aWriteStream.
	analyzer := anAnalyzer.! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:11'!
instancesStep

	clusters do: [ :aCluster | aCluster registerIndexesOn: objectsIndexes ].
	clusters do: [ :aCluster | self clusterInstancesStepOf: aCluster ]! !

!FLSerialization methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/27/2011 17:18'!
objectsIndexes
	^ objectsIndexes ! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:11'!
referencesStep

	clusters do: [ :aCluster | aCluster serializeReferencesStepWith: self ]! !

!FLSerialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:33'!
root
	^ root! !

!FLSerialization methodsFor: 'serializing' stamp: 'MarianoMartinezPeck 12/9/2011 20:34'!
run
	"Serialize the graph starting at the root object."

	self analysisStep.
	self headerStep.
	self instancesStep.
	self referencesStep.
	self trailerStep.! !

!FLSerialization methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 12/5/2011 23:02'!
serializedObjectsIncludes: anObject
	^ objectsIndexes includesKey: anObject! !

!FLSerialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:11'!
stream
	^ stream! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:11'!
trailerStep

	self encodeReferenceTo: root! !

Object subclass: #FLSerializer
	instanceVariableNames: 'analyzer streamFactory serializationFactory streamFinalizer signature version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLSerializer commentStamp: 'MartinDias 8/29/2011 19:10' prior: 0!
I am a binary object serializer.

An example of use:

| sourceArray loadedArray |
sourceArray := 
	Array 
		with: 'a string' 
		with: Transcript
		with: [ Transcript show: 'a string' ].

"Store to the file"
FLSerializer serialize: sourceArray toFileNamed: 'example.FL'. 

"Load from the file"
loadedArray := FLMaterializer materializeFromFileNamed: 'example.FL'. 

"The arrays are not the same"
[ sourceArray ~~ loadedArray ] assert.

"The strings are not the same"
[ sourceArray first ~~ loadedArray first ] assert.
[ sourceArray first = loadedArray first ] assert.

"The global instance Transcript is the same"
[ sourceArray second == loadedArray second ] assert.

"Appreciate in Transcript that the loaded block prints a string"
loadedArray third value.
!

!FLSerializer class methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/9/2011 14:26'!
currentVersion
	"If you change this method, you should also create a version in ConfigurationOfFuel and FLMaterializer >> currentVersion"
	^ 17! !

!FLSerializer class methodsFor: 'protected' stamp: 'MartinDias 10/6/2011 23:28'!
defaultSignature
	^ 'FUEL'! !

!FLSerializer class methodsFor: 'instance creation' stamp: 'MartinDias 9/25/2011 20:12'!
newDefault

	^self newLight! !

!FLSerializer class methodsFor: 'instance creation' stamp: 'MartinDias 9/25/2011 20:12'!
newLight

	^self new! !

!FLSerializer class methodsFor: 'serializing-shortcuts' stamp: 'MartinDias 9/9/2011 18:25'!
serialize: anObject toFileNamed: aFilename

	StandardFileStream 
		fileNamed: aFilename 
		do: [ :aFileStream | 
			self newDefault serialize: anObject on: aFileStream binary ]! !

!FLSerializer class methodsFor: 'serializing-shortcuts' stamp: 'MartinDias 10/12/2011 11:58'!
serializeToByteArray: anObject
	"No stream is needed by the user. An internal in-memory stream will be used. This method returns a ByteArray representing the serialization"
	
	| aStream |
	aStream := (ByteArray new: 100) writeStream.
	
	self newDefault serialize: anObject on: aStream.
	
	^aStream contents! !

!FLSerializer methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 10/3/2011 16:18'!
analyzer
	analyzer isNil ifTrue: [ analyzer := self defaultAnalyzer ].
	^ analyzer! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 17:33'!
analyzer: anObject
	analyzer := anObject! !

!FLSerializer methodsFor: 'protected' stamp: 'MartinDias 9/13/2011 19:26'!
defaultAnalyzer

	^FLAnalyzer newDefault! !

!FLSerializer methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/3/2011 16:29'!
defaultSerialization

	^ serializationFactory := [:anObject :aStream | 
		(FLSerialization 
			root: anObject 
			on: aStream
			analyzer: self analyzer)
			run;
			yourself ]! !

!FLSerializer methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:41'!
fuelAccept: aVisitor  
	^ aVisitor visitSubstitution: self! !

!FLSerializer methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 12/9/2011 20:42'!
fuelSubstitution 
	^ nil! !

!FLSerializer methodsFor: 'initialize-release' stamp: 'MartinDias 10/6/2011 23:29'!
initialize

	super initialize.
	self writeBuffered.
	self signature: self class defaultSignature.
	self version: self class currentVersion.! !

!FLSerializer methodsFor: 'protected' stamp: 'MartinDias 10/6/2011 23:34'!
nextPutSignatureOn: aStream
	aStream nextPutAll: self signature asByteArray! !

!FLSerializer methodsFor: 'protected' stamp: 'MartinDias 10/6/2011 23:34'!
nextPutVersionOn: aStream
	aStream uint16: self version
! !

!FLSerializer methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 10/3/2011 16:29'!
serializationFactory
	serializationFactory isNil ifTrue: [ serializationFactory := self defaultSerialization ].
	^ serializationFactory! !

!FLSerializer methodsFor: 'serializing' stamp: 'MarianoMartinezPeck 11/30/2011 11:53'!
serialize: anObject on: destinationStream 
	"Serialize the graph starting at the root object received and answers the FLSerialization object"

	| aStream |
	aStream := streamFactory value: destinationStream.
	[	self nextPutSignatureOn: aStream.
		self nextPutVersionOn: aStream.
		^ self serializationFactory value: anObject value: aStream ] ensure: [ 
		streamFinalizer value: aStream ]
! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 22:56'!
signature
	^ signature! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 22:56'!
signature: anObject
	signature := anObject! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 22:56'!
version
	^ version! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 22:56'!
version: anObject
	version := anObject! !

!FLSerializer methodsFor: 'configuring-stream' stamp: 'MartinDias 9/9/2011 20:18'!
writeBuffered

	self writeBufferedSize: 4096  ! !

!FLSerializer methodsFor: 'configuring-stream' stamp: 'MartinDias 9/27/2011 20:28'!
writeBufferedSize: aSize

	streamFactory := [:aStream | 
		(FLBufferedWriteStream on: aStream)
			sizeBuffer: aSize;
			yourself ].
	streamFinalizer := [:aBufferedStream | aBufferedStream flush ].! !

Object subclass: #FLSimpleStack
	instanceVariableNames: 'array slotIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Collections'!

!FLSimpleStack commentStamp: 'MarianoMartinezPeck 6/5/2011 12:29' prior: 0!
FLSimpleStack is a simple stack used in Fuel to avoid a recursion while traversing the graph to serialize. While analizing the content of an object (inst vars) we can encounter simple objects (direct string representation) or complex objects (composite). In the latter case we start analizing those instVars.

So... Imagine object X with two instVars Y and Z. Imagine the method of the traverse is called #analye:.  So you do #analize: X.  Then, in such method you check whether X has regular pointers to regular objects, and if true, you analize them. So in this case you would send #analyze: Y and #analize: Z, generating the recursion. Right ?  Notice that the time between analyse: Y and analyse: Z is dependent on the graph that is attached to Y. Y can have objects that have further objects etc. So leaving X aside, Y can be the first object to serialize and Z the last.

With this FLSimpleStack, instead of sending #analyze: Y and #analize: Z  what Fuel does is just to do a push on a stack:   #push: Y  and #push: Z. And then we are done with X, so we pop from the stack and we continue with the next object (at some point in the future we will pop Y and Z) When there are no more objects in the stack it means we are done. !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:11'!
hintSize

	^64! !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:11'!
new
	
	^ self new: self hintSize! !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:09'!
new: nElements
	"Create a Set large enough to hold nElements without growing"
	^ self basicNew initialize: nElements! !

!FLSimpleStack methodsFor: 'accessing' stamp: 'MartinDias 5/15/2011 23:08'!
capacity

	^ array size! !

!FLSimpleStack methodsFor: 'adding' stamp: 'MartinDias 5/15/2011 23:15'!
grow
	"Become larger in capacity."
	| newArray |
	newArray := Array new: array size * 2.
	newArray replaceFrom: 1 to: array size with: array startingAt: 1.
	array := newArray! !

!FLSimpleStack methodsFor: 'initialize-release' stamp: 'MartinDias 5/15/2011 23:10'!
initialize: hintSize

	array := Array new: hintSize.
	slotIndex := 1.! !

!FLSimpleStack methodsFor: 'accessing' stamp: 'MartinDias 5/15/2011 22:54'!
isEmpty

	^ 1 = slotIndex! !

!FLSimpleStack methodsFor: 'removing' stamp: 'MartinDias 5/15/2011 22:52'!
pop
	"Returns the first element and remove it from the stack."

	slotIndex := slotIndex - 1.	
	^ array at: slotIndex ! !

!FLSimpleStack methodsFor: 'printing' stamp: 'MartinDias 6/3/2011 20:25'!
printOn: aStream 
	"Append a sequence of characters that identify the receiver to aStream."

	super printOn: aStream.
	array printElementsOn: aStream
! !

!FLSimpleStack methodsFor: 'adding' stamp: 'MartinDias 5/15/2011 22:44'!
push: anObject 
	"Adds a new object of any kind on top of the stack."

	array at: slotIndex put: anObject.
	array size = slotIndex ifTrue: [ self grow ].
	slotIndex := slotIndex + 1.
	^ anObject.! !

Object subclass: #FLVariablesMapping
	instanceVariableNames: 'mapping theClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLVariablesMapping commentStamp: 'MartinDias 8/1/2011 03:01' prior: 0!
I am used to materialize instance variables in an object, tolerating "class shape changing".

Cases tolerated are:
- instance variable added
- instance variable order change
- instance variable removed
!

!FLVariablesMapping class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 10/24/2011 16:45'!
for: aClass
	
	^ self basicNew
		initializeWith: aClass;
		yourself.! !

!FLVariablesMapping methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/24/2011 16:46'!
initializeWith: aClass
	super initialize.
	theClass := aClass! !

!FLVariablesMapping methodsFor: 'private' stamp: 'MartinDias 7/27/2011 23:21'!
instanceVariableNamesToSerialize

	| ignoredInstanceVariableNames instanceVariableNamesToSerialize |
	ignoredInstanceVariableNames := theClass fuelIgnoredInstanceVariableNames.
	instanceVariableNamesToSerialize := OrderedCollection new.

	theClass fuelInstVarNamesAndOffsetsDo: [:name :index | 
		(ignoredInstanceVariableNames includes: name) 
			ifFalse: [ instanceVariableNamesToSerialize add: name ]].

	^ instanceVariableNamesToSerialize! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 23:22'!
materializeMappingFrom: aStream

	| instanceVariableNamesSerialized |
	instanceVariableNamesSerialized := (1 to: aStream next) collect: [ :index | aStream nextString ].

	mapping := instanceVariableNamesSerialized  
		collect: [ :name | theClass instVarIndexFor: name ifAbsent: [ nil ] ].

! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 20:37'!
materializeReferencesOf: anObject with: aMaterializer 

	mapping do: [ :index | 
			| reference |
			reference := aMaterializer nextEncodedReference.
			index ifNotNil: [
				anObject 
					instVarAt: index
					put: reference ]].! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 23:22'!
serializeMappingOn: aStream

	| instanceVariableNamesToSerialize |
	instanceVariableNamesToSerialize := self instanceVariableNamesToSerialize.

	aStream nextPut: instanceVariableNamesToSerialize size.	
	instanceVariableNamesToSerialize do: [ :name | aStream nextStringPut: name ].

	mapping := instanceVariableNamesToSerialize 
		collect: [ :name | theClass instVarIndexFor: name ].! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 20:02'!
serializeReferencesOf: anObject with: aSerializer 

	mapping do: [ :index | 
		aSerializer encodeReferenceTo: (anObject instVarAt: index) ].! !

Object subclass: #FuelUtilities
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Utilities'!

!FuelUtilities commentStamp: 'MarianoMartinezPeck 6/5/2011 12:25' prior: 0!
FuelUtilities is a class that contains utilities when developing Fuel. Right now the only usage is to do some cleanings. Sometimes updating versions from Monticello bring problems with traits or things like that. Right now the only usage is:

FuelUtilities cleanUpEverything.
!

!FuelUtilities classSide methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 12/17/2011 00:05'!
cleanUpEverything
	"Sometimes when updating Fuel versions Monticello gets stupid and we need to do some cleanups."
	3 timesRepeat: [Smalltalk garbageCollect].
	Smalltalk fixObsoleteReferences.

	(PackageInfo named: 'Fuel') classes,
	(PackageInfo named: 'FuelFameExtension') classes,
	(PackageInfo named: 'FuelContainer') classes,
	(PackageInfo named: 'FuelMetalevel') classes,
	(PackageInfo named: 'FuelMooseExtension') classes
	    do: [:c | c compileAll.  
		 c isBehavior ifTrue: [ c initialize ]
		 ].! !

!FuelUtilities classSide methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 11/6/2011 17:14'!
exportFuelBasic
	| gofer |
	
	"Identify the source repository"
	gofer := Gofer new squeaksource: 'Fuel';
		package: 'ConfigurationOfFuel' yourself.
	"Traverse all packages defined in version '0.1' of the tODE configuration"
	(ConfigurationOfFuel  project version:  '1.7-baseline') packages do:[:spec |
		"Use the #name and #package: messages and Gofer will fetch 
		all versions of the Monticello package"
		gofer package: spec name ].
	"Initiate the download with the #fetch command"
	gofer fetch.

	"Identify the target repository"
	gofer := Gofer new url: 'http://ss3.gemstone.com/ss/Fuel';
		package: 'ConfigurationOfFuel' yourself.
	"Use the same algorithm for identifying the packages to be pushed"
	(ConfigurationOfFuel project version: '1.7-baseline') packages do:[:spec |
		gofer package: spec name ].
	"Initiate the upload with the #push command"
	gofer push.! !

!FuelUtilities classSide methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 11/6/2011 17:15'!
exportFuelMooseExtensions
	| gofer |
	
	"Identify the source repository"
	gofer := Gofer new squeaksource: 'Fuel';
		package: 'ConfigurationOfFuelMooseExtension' yourself.
	"Traverse all packages defined in version '0.1' of the tODE configuration"
	(ConfigurationOfFuelMooseExtension  project version:  '1.2-baseline') packages do:[:spec |
		"Use the #name and #package: messages and Gofer will fetch 
		all versions of the Monticello package"
		gofer package: spec name ].
	"Initiate the download with the #fetch command"
	gofer fetch.

	"Identify the target repository"
	gofer := Gofer new url: 'http://ss3.gemstone.com/ss/Fuel';
		package: 'ConfigurationOfFuelMooseExtension' yourself.
	"Use the same algorithm for identifying the packages to be pushed"
	(ConfigurationOfFuelMooseExtension project version: '1.2-baseline') packages do:[:spec |
		gofer package: spec name ].
	"Initiate the upload with the #push command"
	gofer push.! !

!FuelUtilities classSide methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 11/6/2011 17:15'!
exportRemainingPackages

	Gofer it
		squeaksource: 'Fuel';
		package: 'FuelFameExtensionBenchmarks';
		package: 'FuelGZip';
		package: 'FuelPresentations';
		package: 'FuelCompatibilityBeforePharo12';
		package: 'FuelCompatibilityBeforePharo13';
		package: 'FuelCompatibilityForSqueak';
	fetch.

	Gofer it
		url: 'http://ss3.gemstone.com/ss/Fuel';
		package: 'FuelFameExtensionBenchmarks';
		package: 'FuelGZip';
		package: 'FuelPresentations';
		package: 'FuelCompatibilityBeforePharo12';
		package: 'FuelCompatibilityBeforePharo13';
		package: 'FuelCompatibilityForSqueak';
	push.! !

!FuelUtilities classSide methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 11/6/2011 17:13'!
exportToSS3
	self exportFuelBasic.
	self exportFuelMooseExtensions.
	self exportRemainingPackages.! !

!Object methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 12/6/2011 00:32'!
fuelAccept: aVisitor

	"Be careful because the order is important. For example, weak are also variable, but we need that weak objects send #visitWeakObject: and not #visitVariableObject: "
	self class isFixed
		ifTrue: [ ^aVisitor visitFixedObject: self ].
	self class isWeak
		ifTrue: [ ^aVisitor visitWeakObject: self ].
	self class isPointers
		ifTrue: [ ^aVisitor visitVariableObject: self ].
	self class isBytes
		ifTrue: [ ^aVisitor visitBytesObject: self ].
	self class isWords
		ifTrue: [	 ^aVisitor visitWordsObject: self ].
	self error: 'Something is wrong!!'! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 8/27/2011 19:03'!
fuelAfterMaterialization
	"Materialization process will send this message after materializing instances of my class"! !
FLLightGlobalMapper initialize!
FLWellKnownObjectsCluster initialize!
LargeIdentityHashedCollection initialize!
